SystemOrganization addCategory: #'OB-Standard-Actors'!
SystemOrganization addCategory: #'OB-Standard-Browsers'!
SystemOrganization addCategory: #'OB-Standard-Definitions'!
SystemOrganization addCategory: #'OB-Standard-Nodes'!
SystemOrganization addCategory: #'OB-Standard-Tests'!
SystemOrganization addCategory: #'OB-Standard-Utilities'!

OBActor subclass: #OBCategoryActor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Actors'!

!OBCategoryActor commentStamp: 'cwp 1/7/2005 23:43' prior: 0!
OBCategory actor implements a number of actions useful for manipulating both class and method categories.!

!OBCategoryActor methodsFor: 'queries' stamp: ''!
actionsForNode: aNode
	^ Array
		with: (OBAction
				label: 'remove'
				receiver: self
				selector: #remove:
				arguments: {aNode}
				keystroke: $x
				icon: MenuIcons deleteIcon)
		with: (OBAction
				label: 'rename...'
				receiver: self
				selector: #rename:
				arguments: {aNode})! !

!OBCategoryActor methodsFor: 'queries' stamp: 'cwp 1/9/2005 11:30'!
actionsForParent: aNode
	| actions |
	actions _ self stdActionsForParent: aNode.
	aNode organization 
			isClassOrganizer ifTrue: [actions _ actions copyWith: (self categorizeActionFor: aNode)].
	^ actions! !

!OBCategoryActor methodsFor: 'actions' stamp: ''!
alphabetizeCategoriesIn: anOrganizer 
	anOrganizer sortCategories.
	OBListChanged signal! !

!OBCategoryActor methodsFor: 'queries' stamp: 'cwp 9/18/2004 23:52'!
categorizeActionFor: aNode
	^ OBAction
		label: 'categorize automatically'
		receiver: self
		selector: #categorizeIn:
		arguments: (Array with: aNode organization)! !

!OBCategoryActor methodsFor: 'actions' stamp: 'cwp 9/18/2004 22:53'!
categorizeIn: aClassOrganizer 
	"Categorize methods by looking in parent classes for a method category."
	| organizers |
	organizers := aClassOrganizer subject withAllSuperclasses collect: [:ea | ea organization].
	(aClassOrganizer listAtCategoryNamed: ClassOrganizer default) do: [:sel | | found |
		found := (organizers collect: [ :org | org categoryOfElement: sel])
			detect: [:ea | ea ~= ClassOrganizer default and: [ ea ~= nil]]
			ifNone: [].
		found ifNotNil: [aClassOrganizer classify: sel under: found]]! !

!OBCategoryActor methodsFor: 'private' stamp: 'cwp 9/23/2004 23:29'!
categoryTemplateFor: anOrganization
	^ anOrganization isClassOrganizer
		ifTrue: ['category name']
		ifFalse: ['Category-Name']! !

!OBCategoryActor methodsFor: 'actions' stamp: 'cwp 9/23/2004 23:28'!
createIn: anOrganization
	| catName |
	catName := OBTextRequest 
					prompt: 'Please type new category name' 
					template: (self categoryTemplateFor: anOrganization).
	catName ifNotNil:	[anOrganization addCategory: catName.
						(OBClassCategoryNode on: catName) signalSelection]! !

!OBCategoryActor methodsFor: 'actions' stamp: 'cwp 9/18/2004 21:55'!
removeEmptyCategoriesIn: anOrganizer 
	anOrganizer removeEmptyCategories! !

!OBCategoryActor methodsFor: 'actions' stamp: 'cwp 9/19/2004 00:11'!
remove: aNode
	| list choice |
	list := aNode container organization listAtCategoryNamed: aNode name.
	list isEmpty ifTrue: [^ aNode remove].
	choice := OBConfirmationRequest prompt: 'Are you sure you want to
remove this category 
and all its elements?' confirm: 'Remove'.
	choice ifTrue: [^ aNode remove]! !

!OBCategoryActor methodsFor: 'actions' stamp: ''!
rename: aNode
	| category |
	category := OBTextRequest
				prompt: 'Please type new category name' 
				template: aNode name.
	category ifNotNil:	
		[aNode container organization renameCategory: aNode name toBe: category.
		OBListChanged signal]! !

!OBCategoryActor methodsFor: 'actions' stamp: 'cwp 9/22/2004 21:09'!
reorganizeCategoriesIn: anOrganizer
	^ OBDefinitionChanged definition: (OBOrganizationDefinition on: anOrganizer)! !

!OBCategoryActor methodsFor: 'queries' stamp: 'cwp 9/22/2004 20:58'!
stdActionsForParent: aNode
	^ Array
		with: (OBAction 
				label: 'create category...'
				receiver: self
				selector: #createIn:
				arguments: (Array with: aNode organization)
				icon: MenuIcons newIcon)
		with: (OBAction
				label: 'alphabetize categories'
				receiver: self
				selector: #alphabetizeCategoriesIn:
				arguments: (Array with: aNode organization))
		with: (OBAction
				label: 'remove empty categories'
				receiver: self
				selector: #removeEmptyCategoriesIn:
				arguments: (Array with: aNode organization))
		with: (OBAction
				label: 'reorganize categories'
				receiver: self
				selector: #reorganizeCategoriesIn:
				arguments: (Array with: aNode organization))
! !

OBActor subclass: #OBClassActor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Actors'!

!OBClassActor commentStamp: 'cwp 1/7/2005 23:44' prior: 0!
OBClassActor provides basic actions for manipulating classes.!

!OBClassActor methodsFor: 'querying' stamp: 'cwp 12/2/2004 23:09'!
actionsForNode: aClassNode
	^ {OBAction
			label: 'remove' 
			receiver: self
			selector: #remove:
			arguments: {aClassNode}
			keystroke: $x 
			icon: MenuIcons deleteIcon.
		OBAction
			label: 'rename...' 
			receiver: self
			selector: #rename:
			arguments: {aClassNode}.
		OBAction
			label: 'copy...' 
			receiver: self
			selector: #copy:
			arguments: {aClassNode}.
		OBAction
			label: 'subclass template'
			receiver: self
			selector: #subclassTemplate:
			arguments: {aClassNode}.
		OBAction
			label: 'browse references'
			receiver: self
			selector: #browseReferences:
			arguments: {aClassNode}
	}! !

!OBClassActor methodsFor: 'private' stamp: 'avi 9/17/2005 01:21'!
browseObsoleteRefs: aClassNode as: oldName 
	| binding |
	binding := aClassNode theClass environment 
				associationAt: aClassNode theClass name.
	(SystemNavigation default allCallsOn: binding) isEmpty 
		ifFalse: [OBReferencesBrowser
					browseRoot: aClassNode
					title: 'Obsolete references to']! !

!OBClassActor methodsFor: 'actions' stamp: 'avi 9/17/2005 01:34'!
browseReferences: aClassNode 
	OBReferencesBrowser browseRoot: aClassNode copy! !

!OBClassActor methodsFor: 'private' stamp: ''!
copyClass: oldClass as: newName
	| oldDefinition newDefinition newClass |
	(oldClass environment hasClassNamed: newName)
		ifTrue: [^self error: newName, ' already exists'].
	oldDefinition := oldClass definition.
	newDefinition := oldDefinition 
						copyReplaceAll: '#' , oldClass name asString 
						with: '#' , newName asString.
	Cursor wait 
		showWhile: [newClass := Compiler evaluate: newDefinition logged: true.
					newClass copyAllCategoriesFrom: oldClass.
					oldClass class copyAllCategoriesFrom: oldClass class].
	^ newClass! !

!OBClassActor methodsFor: 'actions' stamp: ''!
copy: aClassNode
	| newName newClass |
	newName := OBTextRequest
			prompt: 'Please type new class name'
			template: aClassNode theNonMetaClassName asString.
	newName ifNotNil:	[newClass := self
										copyClass: aClassNode theNonMetaClass 
										as: newName asSymbol.
						(OBClassNode on: newClass) signalSelection].! !

!OBClassActor methodsFor: 'actions' stamp: ''!
remove: aClassNode
	(OBConfirmationRequest
		prompt: 'Are you certain that you 
want to REMOVE the class ', aClassNode theClassName, ' from the system?'
		confirm: 'Remove')
			ifTrue: [aClassNode theClass removeFromSystem.
					aClassNode signalDeletion]! !

!OBClassActor methodsFor: 'actions' stamp: 'cwp 11/30/2004 22:05'!
rename: aClassNode
	| newName |
	newName := OBTextRequest
					prompt: 'Please type new class name' 
					template: aClassNode theClassName asString.
	newName ifNotNil:	[ | oldName |
						oldName := aClassNode theNonMetaClass name.
						aClassNode theClass environment 
							renameClassNamed: oldName
							as: newName asSymbol.
						self browseObsoleteRefs: aClassNode as: oldName].
	! !

!OBClassActor methodsFor: 'actions' stamp: 'cwp 9/17/2004 00:15'!
subclassTemplate: aClassNode 
	| class |
	class _ aClassNode theClass.
	OBDefinitionChanged definition: 
		(OBClassDefinition 
			environment: class environment 
			template: (Class templateForSubclassOf: class category: class category))! !

OBBrowserTest subclass: #OBAccessBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBAccessBrowserTest methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:39'!
browserClass
	^ OBVariablesBrowser! !

!OBAccessBrowserTest methodsFor: 'configuration' stamp: ''!
node
	^ OBClassNode on: BogusB! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: ''!
testInitalDisplay
	self assertListAt: 1 contains: BogusInfo bInstVarNames! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: ''!
testSelectCVar
	self clickButtonLabeled: 'class'.
	self select: 'FirstClassVariableB'.
	self assertListAt: 2 contains: #('BogusB>>accessesCVar' 
								   'BogusB class>>accessesCVarAlso')! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: ''!
testSelectIVarAccessor
	self select: 'firstInstanceVariableB'.
	self select: 'BogusB>>accessesIvar'.
	self assertListAt: 3 includes: #('BogusB>>sendsAccessesIvar')! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: ''!
testSelectIvar
	self select: 'firstInstanceVariableB'.
	self assertListAt: 2 contains: #('BogusB>>accessesIvar')! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: ''!
testSelectSender
	self select: 'firstInstanceVariableB'.
	self select: 'BogusB>>accessesIvar'.
	self select: 'BogusB>>sendsAccessesIvar'.
	self assertListAt: 4 contains: #('BogusB>>sendsSendsAccessesIvar')
! !

OBBrowserTest subclass: #OBChasingBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBChasingBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/8/2004 22:59'!
isAbstract
	^ self name = #OBChasingBrowserTest! !

!OBChasingBrowserTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 23:03'!
testChaseDisplay
	self clickButtonLabeled: 'chase'.
	self 
		assertListAt: 1 
		contains: self initialList! !

!OBChasingBrowserTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 22:58'!
testInitialDisplay
	self 
		assertListAt: 1 
		contains: self initialList! !

OBChasingBrowserTest subclass: #OBImplementorsBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBImplementorsBrowserTest methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:40'!
browserClass
	^ OBImplementorsBrowser! !

!OBImplementorsBrowserTest methodsFor: 'configuration' stamp: 'cwp 12/8/2004 23:37'!
initialList
	^ #('BogusB>>sendsSendsAccessesIvar')! !

!OBImplementorsBrowserTest methodsFor: 'configuration' stamp: 'cwp 12/8/2004 23:37'!
node
	"We don't want this method to show up in senders 
	of #sendsSendsAccesses, since other tests 
	aren't expecting it."
	
	| selector |
	selector := ('sendsSendsAccesses', 'Ivar') asSymbol.
	^ OBMethodNode on: selector inClass: BogusB! !

!OBImplementorsBrowserTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 23:41'!
testMessagesOf
	self clickButtonLabeled: 'chase'.
	self select: 'BogusB>>sendsSendsAccessesIvar'.
	self assertListAt: 2 contains: #('sendsAccessesIvar')! !

OBChasingBrowserTest subclass: #OBSendersBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBSendersBrowserTest methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:40'!
browserClass
	^ OBSendersBrowser! !

!OBSendersBrowserTest methodsFor: 'configuration' stamp: 'cwp 12/8/2004 22:59'!
initialList
	^  #('BogusB>>sendsAccessesIvar')! !

!OBSendersBrowserTest methodsFor: 'configuration' stamp: 'cwp 12/8/2004 22:44'!
node
	"We don't want this method to show up in senders 
	of #accessesIvar, since that's what we're testing"
	
	| selector |
	selector := ('accesses', 'Ivar') asSymbol.
	^ OBMethodNode on: selector inClass: BogusB! !

!OBSendersBrowserTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 23:15'!
testSendersOf
	self clickButtonLabeled: 'chase'.
	self select: 'BogusB>>sendsAccessesIvar'.
	self assertListAt: 2 contains: #('BogusB>>sendsSendsAccessesIvar')! !

OBBrowserTest subclass: #OBCodeBrowserTest
	instanceVariableNames: 'damage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBCodeBrowserTest class methodsFor: 'as yet unclassified' stamp: ''!
isAbstract
	^ self name = #OBCodeBrowserTest! !

!OBCodeBrowserTest class methodsFor: 'as yet unclassified' stamp: ''!
resources
	^ Array with: BogusSnapshotResource! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
altMethodA
	^ 'a
	^ $B'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
altMethodOne
	^ 'one
	^ 11'! !

!OBCodeBrowserTest methodsFor: 'asserting' stamp: ''!
assertIsClassSide
	self assertButtonSelected: 'class'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
bogusAClassTemplate
	^  'BogusA class
	instanceVariableNames: '''''! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 12/12/2004 23:51'!
bogusAName
	^'BogusA'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
bogusATemplate
	^ 'Object subclass: #BogusA
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	package: ''', BogusInfo packageName, '''
	category: ''Base'''! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
bogusBComment
	^ 'A comment for BogusB'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 12/12/2004 23:40'!
bogusCategoryName
	^ BogusInfo baseCategory! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
bogusCDefinition
 ^ 'Object subclass: #BogusC
	instanceVariableNames: ''foo''
	classVariableNames: ''''
	poolDictionaries: ''''
	package: ''', self bogusPackageName, '''
	category: ''Base'''! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 12/10/2004 21:15'!
bogusClassNames
	^BogusInfo baseClassNames! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
bogusDDefinition
 ^ 'Object subclass: #BogusD
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	package: ''', BogusInfo packageName, '''
	category: ''Base'''! !

!OBCodeBrowserTest methodsFor: 'running' stamp: ''!
browserResource
	^ self class resources first current! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
classTemplate
	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	package: ''', BogusInfo packageName, '''
	category: ''Base'''! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: ''!
deselectBogusCategory
	self deselect: self bogusCategoryName! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
fakeAClassTemplate
	^  self deprecated: 'Use #bogusAClassTemplate'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
fakeAComment
	self deprecated: 'Use BogusInfo>>aComment instead'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
fakeATemplate
	self deprecated: 'Use #bogusATemplate instead'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
fakeClassBComment
	self deprecated: 'Use #bogusBComment'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
fakeClassCDefinition
	self deprecated: 'Use #bogusCDefinition'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
fakeClassDDefinition
	self deprecated: 'Use #bogusDDefinition'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
fakePackageName
	self deprecated: 'Use BogusInfo>>packageName'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
methodA
	^ 'a
	^ $A'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
methodFour
	^ 'four
	^ 4'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
methodOne
	^ 'one
	^ 1'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: ''!
methodTemplate
	^ 'message selector and argument names
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!OBCodeBrowserTest methodsFor: 'running' stamp: ''!
runCase
	SystemChangeNotifier uniqueInstance doSilently: [super runCase]! !

!OBCodeBrowserTest methodsFor: 'running' stamp: ''!
runCaseAsFailure: aSemaphore
	SystemChangeNotifier uniqueInstance doSilently: [super runCaseAsFailure: aSemaphore]! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/13/2004 00:01'!
selectBogusA
	self select: self bogusAName! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: ''!
selectBogusCategory
	self select: self bogusCategoryName! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: ''!
selectBogusPackage
	! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 12/10/2004 21:32'!
setUp
	damage := BogusDamage new.
	model _ self browserClass metaNode: self metagraph node: self node.
	morph _ model morph.
	self forceLayout! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: ''!
switchToClassSide
	self clickButtonLabeled: 'class'! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: ''!
switchToComment
	self clickButtonLabeled: '?'! !

!OBCodeBrowserTest methodsFor: 'running' stamp: ''!
tearDown
	damage repair! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:01'!
testButtonRemoval
	self selectBogusCategory.
	self selectBogusA.
	self deselectBogusCategory.
	self assertNoButtonsVisible! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:26'!
testColumnClearing
	self selectBogusCategory.
	self selectBogusA.
	self select: 'numbers'.
	self select: 'one'.
	self switchToClassSide.
	self assertAListIncludes: self bogusClassNames.
	self assertAListIncludes: #('testing' 'capitals').
	self denyAListIncludes: #('letters' 'numbers').
	self denyAListIncludes: #('one' 'two' 'three').
	self assertTextIs: self bogusAClassTemplate! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: ''!
testCreateClassFromClass
	self selectBogusCategory.
	self select: 'BogusC'.
	self setText: self bogusDDefinition.
	damage add: #Definition ofClass: #BogusD.
	self assertAListIncludes: #(BogusD).
	self assertCurrentItemIs: #BogusD.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:09'!
testCreateClassFromTemplate
	self selectBogusCategory.
	damage add: #Definition ofClass: #BogusD.
	self setText: self bogusDDefinition.
	self assertAListIncludes: #(BogusD).
	self assertCurrentItemIs: #BogusD ! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/20/2004 00:37'!
testCreateMethodFromMethod
	self selectBogusCategory.
	self selectBogusA.
	self select: 'numbers'.
	self select: 'one'.
	damage add: #four ofClass: #BogusA.
	self setText: self methodFour.
	self assertAListMatches: #('one' 'two' 'three' 'four' 'five').
	self assertCurrentItemIs: 'four' 
! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:04'!
testCreateMethodFromTemplate
	self selectBogusCategory.
	self selectBogusA.
	self select: 'letters'.
	self setText: self methodFour.
	damage add: #four ofClass: #BogusA.
	self assertAListMatches: #('a' 'b' 'four').
	self assertCurrentItemIs: 'four' 
! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:04'!
testDeselectFakeA
	self selectBogusCategory.
	self selectBogusA.
	self deselect: self bogusAName.
	self assertAListIncludes: {self bogusCategoryName}.
	self assertAListIncludes: self bogusClassNames.
	self denyAListIncludes: #('letters' 'numbers').
	self assertTextIs: self classTemplate.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/10/2004 23:46'!
testEditClass
	self selectBogusCategory.
	self select: 'BogusC'.
	self setText: self bogusCDefinition.
	damage add: #Definition ofClass: #BogusC.
	self assertAListIncludes: self bogusClassNames.
	self assertCurrentItemIs: #BogusC.
	self assert: BogusC instVarNames = #(foo)! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:04'!
testEditClassMethod
	self selectBogusCategory.
	self selectBogusA.
	self switchToClassSide.
	self select: 'capitals'.
	self select: 'a'.
	self setText: self altMethodA.
	damage add: #a ofClass: #BogusA isMeta: true.
	self assertAListMatches: #('a' 'b').
	self assertCurrentItemIs: 'a'.
	self assert: $B = BogusA a! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/15/2004 21:06'!
testEditComment
	self selectBogusCategory.
	self selectBogusA.
	self switchToComment.
	self setText: self bogusBComment.
	damage add: #Comment ofClass: #BogusA.
	self assertAListIncludes: self bogusClassNames.
	self assertCurrentItemIs: #BogusA.
	self assert: BogusA comment = self bogusBComment! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:04'!
testEditMethod
	self selectBogusCategory.
	self selectBogusA.
	self select: 'numbers'.
	self select: 'one'.

	damage add: #one ofClass: #BogusA.
	self setText: self altMethodOne.
	self assertAListMatches: #('one' 'two' 'three' 'five').
	self assertCurrentItemIs: 'one'.
	self assert: 11 = BogusA new one! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/12/2004 23:55'!
testJumpToBogusA
	self selectBogusPackage.
	model jumpTo: (OBClassNode on: BogusA).
	self assertCurrentItemIs: self bogusAName! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: ''!
testJumpToCapitals
	self selectBogusPackage.
	model jumpTo: (OBMethodCategoryNode 
					on: 'capitals' 
					inClass: BogusA class).
	self assertCurrentItemIs: 'capitals'.
	self assertIsClassSide.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/12/2004 23:55'!
testJumpToFakes
	self selectBogusPackage.
	model jumpTo: (OBClassCategoryNode 
						on: self bogusCategoryName 
						inEnvironment: Smalltalk).
	self assertCurrentItemIs: self bogusCategoryName! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: ''!
testJumpToMethodOne
	self selectBogusPackage.
	model jumpTo: (OBMethodNode 
					on: #one 
					inClass: BogusA).
	self assertCurrentItemIs: 'one'! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: ''!
testJumpToNumbers
	self selectBogusPackage.
	model jumpTo: (OBMethodCategoryNode 
					on: 'numbers' 
					inClass: BogusA).
	self assertCurrentItemIs: 'numbers'.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:04'!
testMethodChangedElsewhere
	self selectBogusCategory.
	self selectBogusA.
	self select: 'numbers'.
	self select: 'one'.
	self editText: 'foo'.

	damage add: #one ofClass: #BogusA.
	BogusA compile: self altMethodOne.
	model refresh.

	self assertAListMatches: #('one' 'two' 'three' 'five').
	self assertCurrentItemIs: 'one'.
	self assert: self textMorph hasEditingConflicts
	! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:25'!
testSelectAllMethodCategory
	self selectBogusCategory.
	self selectBogusA.
	self select: '-- all --'.
	self assertAListIncludes: self bogusClassNames.
	self assertAListIncludes: #('letters' 'numbers').
	self assertAListIncludes: #('one' 'two' 'three' 'five' 'a' 'b').
	self assertTextIs: self methodTemplate! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:26'!
testSelectBogusA
	self selectBogusCategory.
	self selectBogusA.
	self assertAListIncludes: self bogusClassNames.
	self assertAListIncludes: #('letters' 'numbers').
	self assertCurrentItemIs: self bogusAName.
	self assertTextIs: self bogusATemplate! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:23'!
testSelectBogusAClassSide
	self selectBogusCategory.
	self selectBogusA.
	self switchToClassSide.
	self assertAListIncludes: self bogusClassNames.
	self assertAListIncludes: #('capitals').
	self assertCurrentItemIs: self bogusAName.
	self assertTextIs: self bogusAClassTemplate! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:23'!
testSelectBogusAComment
	self selectBogusCategory.
	self selectBogusA.
	self switchToComment.
	self assertAListIncludes: self bogusClassNames.
	self denyAListIncludes: #('letters' 'numbers').
	self assertCurrentItemIs: self bogusAName.
	self assertTextIs: BogusInfo aComment.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/10/2004 21:16'!
testSelectCategory
	self selectBogusCategory.
	self assertAListIncludes: {self bogusCategoryName}.
	self assertAListIncludes: self bogusClassNames.
	self assertTextIs: self classTemplate! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:26'!
testSelectNumbers
	self selectBogusCategory.
	self selectBogusA.
	self select: 'numbers'.
	self assertAListIncludes: self bogusClassNames.
	self assertAListIncludes: #('letters' 'numbers').
	self assertAListIncludes: #('one' 'two' 'three').
	self assertTextIs: self methodTemplate.
	self assertTextIsSelected.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/13/2004 00:30'!
testSelectOne
	self selectBogusCategory.
	self selectBogusA.
	self select: 'numbers'.
	self select: 'one'.
	self assertAListIncludes: self bogusClassNames.
	self assertAListIncludes: #('letters' 'numbers').
	self assertAListIncludes: #('one' 'two' 'three').
	self assertTextIs: BogusInfo aOneSource! !

OBCodeBrowserTest subclass: #OBImageBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

OBImageBrowserTest subclass: #OBAltImageBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBAltImageBrowserTest methodsFor: 'constants' stamp: 'cwp 1/8/2005 11:39'!
bogusClassNames
	^ #('BogusA' 'BogusE' 'BogusB' 'BogusC')! !

!OBAltImageBrowserTest methodsFor: 'constants' stamp: 'avi 9/17/2005 01:12'!
metagraph
	^ OBSystemBrowser defaultMetaNode! !

OBImageBrowserTest subclass: #OBHierImageBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBHierImageBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/12/2004 23:32'!
shouldInheritSelectors
	^ true! !

!OBHierImageBrowserTest methodsFor: 'constants' stamp: 'cwp 1/8/2005 11:49'!
bogusAName
	^ 'BogusA'! !

!OBHierImageBrowserTest methodsFor: 'constants' stamp: 'cwp 1/8/2005 11:52'!
bogusClassNames
	^ #('ProtoObject' 'Object' 'BogusA' 'BogusE')! !

!OBHierImageBrowserTest methodsFor: 'constants' stamp: 'cwp 12/13/2004 00:11'!
bogusDDefinition
 ^ 'BogusA subclass: #BogusD
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self bogusCategoryName, ''''! !

!OBHierImageBrowserTest methodsFor: 'constants' stamp: 'cwp 1/8/2005 12:07'!
bogusDName
	^ 'BogusD'! !

!OBHierImageBrowserTest methodsFor: 'constants' stamp: 'cwp 12/13/2004 00:31'!
bogusEDefinition
 ^ 'BogusA subclass: #BogusE
	instanceVariableNames: ''foo''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self bogusCategoryName, ''''! !

!OBHierImageBrowserTest methodsFor: 'constants' stamp: 'cwp 1/8/2005 12:05'!
bogusEName
	^ 'BogusE'! !

!OBHierImageBrowserTest methodsFor: 'set up' stamp: 'avi 9/17/2005 01:55'!
metagraph
	^ OBSystemBrowser hierarchicalMetaNode! !

!OBHierImageBrowserTest methodsFor: 'set up' stamp: 'cwp 12/12/2004 22:47'!
node
	^ OBClassNode on: BogusA! !

!OBHierImageBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/12/2004 23:43'!
selectBogusCategory
	! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/13/2004 00:07'!
testButtonRemoval
	"Not applicable to hierarchy browser"! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/13/2004 00:33'!
testCreateClassFromClass
	self selectBogusCategory.
	self select: self bogusEName.
	damage add: #Definition ofClass: #BogusD.
	self setText: self bogusDDefinition.
	self assertAListIncludes: {self bogusDName}.
	self assertCurrentItemIs: self bogusDName.! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/13/2004 00:15'!
testCreateClassFromTemplate
	"doesn't apply for now"! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/13/2004 00:34'!
testDeselectFakeA
	self selectBogusCategory.
	self selectBogusA.
	self deselect: self bogusAName.
	self assertAListIncludes: self bogusClassNames.
	self denyAListIncludes: #('letters' 'numbers').
	self assertTextIs: '' "for now"! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/13/2004 00:30'!
testEditClass
	self selectBogusCategory.
	self select: self bogusEName.
	damage add: #Definition ofClass: #BogusE.
	self setText: self bogusEDefinition.
	self assertAListIncludes: self bogusClassNames.
	self assertCurrentItemIs: self bogusEName.
	self assert: BogusE instVarNames = #(foo)! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/15/2004 21:13'!
testEditComment
	self selectBogusCategory.
	self selectBogusA.
	self switchToComment.
	damage add: #Comment ofClass: #BogusA.
	self setText: self bogusBComment.
	self assertAListIncludes: self bogusClassNames.
	self assertCurrentItemIs: self bogusAName.
	self assert: BogusA comment = self bogusBComment! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/15/2004 21:13'!
testInitialDisplay
	self listMorphs size = 1.
	self assertListAt: 1 includes: self bogusClassNames.
	self assertTextIs: ''.! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/12/2004 23:24'!
testJumpToFakes
	"Not applicable to the hierarchy browser"! !

!OBHierImageBrowserTest methodsFor: 'tests' stamp: 'cwp 12/12/2004 23:44'!
testSelectCategory
	"Not applicable to the hierarchy browser"! !

!OBImageBrowserTest methodsFor: 'constants' stamp: 'cwp 12/12/2004 23:39'!
bogusATemplate
	^ 'Object subclass: #BogusA
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self bogusCategoryName, ''''! !

!OBImageBrowserTest methodsFor: 'constants' stamp: 'cwp 12/12/2004 23:39'!
bogusCDefinition
 ^ 'Object subclass: #BogusC
	instanceVariableNames: ''foo''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self bogusCategoryName, ''''! !

!OBImageBrowserTest methodsFor: 'constants' stamp: 'cwp 12/12/2004 23:39'!
bogusDDefinition
 ^ 'Object subclass: #BogusD
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self bogusCategoryName, ''''! !

!OBImageBrowserTest methodsFor: 'set up' stamp: 'cwp 12/10/2004 21:27'!
browserClass
	^ OBSystemBrowser! !

!OBImageBrowserTest methodsFor: 'constants' stamp: 'cwp 12/12/2004 23:40'!
classTemplate
	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self bogusCategoryName, ''''! !

!OBImageBrowserTest methodsFor: 'set up' stamp: 'avi 9/17/2005 01:13'!
metagraph
	^ OBSystemBrowser alphabeticalMetaNode! !

!OBImageBrowserTest methodsFor: 'set up' stamp: 'cwp 12/10/2004 21:28'!
node
	^ OBEnvironmentNode forImage! !

!OBImageBrowserTest methodsFor: 'testing' stamp: 'cwp 12/12/2004 23:40'!
testInitialDisplay
	self listMorphs size = 1.
	self assertListAt: 1 includes: {self bogusCategoryName}.
	self assertTextIs: ''.
	! !

OBBrowserTest subclass: #OBInheritanceBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBInheritanceBrowserTest methodsFor: 'running' stamp: 'cwp 12/8/2004 23:49'!
interiorNode
	^ OBMethodNode on: #isString inClass: BogusA! !

!OBInheritanceBrowserTest methodsFor: 'running' stamp: 'cwp 12/8/2004 23:49'!
leafNode
	^ OBMethodNode on: #isString inClass: BogusE! !

!OBInheritanceBrowserTest methodsFor: 'running' stamp: 'avi 9/17/2005 01:11'!
metagraph
	^ OBInheritanceBrowser defaultMetaNode! !

!OBInheritanceBrowserTest methodsFor: 'running' stamp: 'cwp 12/8/2004 23:49'!
rootNode
	^ OBCollectionNode on: {OBMethodNode 
								on: #isString 
								inClass: Object}! !

!OBInheritanceBrowserTest methodsFor: 'running' stamp: 'cwp 11/28/2004 10:22'!
setUp! !

!OBInheritanceBrowserTest methodsFor: 'running' stamp: 'cwp 12/5/2004 18:03'!
setUpWith: targetNode
	[OBInheritanceBrowser openRoot: self rootNode selection: targetNode]
		on: OBBrowseRequest
		do: [:notification | 
			model := notification browser.
			morph := model morph].
	self forceLayout.! !

!OBInheritanceBrowserTest methodsFor: 'testing' stamp: 'cwp 11/28/2004 10:23'!
testInitialDisplay
	self setUpWith: self interiorNode.
	self 
		assertListAt: 1 contains: #('Object>>isString');
		assertListAt: 2 contains: #('BogusA>>isString' 'AbstractString>>isString');
		assertListAt: 3 contains: #('BogusE>>isString');
		assertCurrentItemIs: 'BogusA>>isString'! !

!OBInheritanceBrowserTest methodsFor: 'testing' stamp: 'cwp 11/28/2004 10:24'!
testInitialDisplay2
	self setUpWith: self leafNode.
	self 
		assertListAt: 1 contains: #('Object>>isString');
		assertListAt: 2 contains: #('BogusA>>isString' 'AbstractString>>isString');
		assertListAt: 3 contains: #('BogusE>>isString');
		assertCurrentItemIs: 'BogusE>>isString'! !

OBBrowserTest subclass: #OBReferencesBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBReferencesBrowserTest methodsFor: 'constants' stamp: 'cwp 12/1/2004 01:18'!
bogusCReferences
	^ {	'OBCodeBrowserTest>>testEditClass'.
		'OBReferencesBrowserTest>>node'.
		'OBReferencesBrowserTest>>refersToBogusC' }! !

!OBReferencesBrowserTest methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:43'!
browserClass
	^ OBReferencesBrowser! !

!OBReferencesBrowserTest methodsFor: 'configuration' stamp: 'cwp 11/30/2004 21:07'!
node
	^ OBClassNode on: BogusC
! !

!OBReferencesBrowserTest methodsFor: 'bogus' stamp: 'cwp 12/1/2004 00:58'!
refersToBogusC
	^ BogusC! !

!OBReferencesBrowserTest methodsFor: 'tests' stamp: 'cwp 12/1/2004 00:32'!
testClassNameSelection
	self select: 'OBReferencesBrowserTest>>node'.
	self assertSelectedTextIs: 'BogusC'! !

!OBReferencesBrowserTest methodsFor: 'tests' stamp: 'cwp 11/30/2004 20:31'!
testInitialDisplay
	self assertListAt: 1 contains: self bogusCReferences! !

!OBReferencesBrowserTest methodsFor: 'tests' stamp: 'cwp 12/1/2004 01:09'!
testSelectorSelection
	self select: 'OBReferencesBrowserTest>>refersToBogusC'.
	self deny: self textMorph getSelection = (9 to: 13)! !

OBBrowserTest subclass: #OBVersionsBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBVersionsBrowserTest methodsFor: 'constants' stamp: 'cwp 11/27/2004 21:43'!
expectedHeadings
	^ #('BogusA>>two cwp 3/11/2004 20:07' 'BogusA>>two cwp 3/11/2004 20:06' 'BogusA>>two cwp 3/11/2004 20:06' 'BogusA>>two cwp 3/11/2004 20:07' 'BogusA>>two cwp 3/11/2004 20:06' 'BogusA>>two cwp 3/11/2004 20:06')! !

!OBVersionsBrowserTest methodsFor: 'running' stamp: 'avi 9/17/2005 01:11'!
metagraph
	^ OBVersionBrowser defaultMetaNode! !

!OBVersionsBrowserTest methodsFor: 'running' stamp: ''!
node
	^ OBMethodNode on: #two inClass: BogusA! !

!OBVersionsBrowserTest methodsFor: 'running' stamp: 'cwp 12/5/2004 18:02'!
setUp
	(BogusA compiledMethodAt: #two) setSourcePointer: self twoSourcePointer.
	[OBVersionBrowser openOn: self node]
		on: OBBrowseRequest, OBSourceFilesRequest 
		do: [:notification |
			notification isBrowseRequest
				ifTrue: [model := notification browser.
						morph := model morph]
				ifFalse: [notification resume: self sourceFiles]].
	self forceLayout
		! !

!OBVersionsBrowserTest methodsFor: 'emulating' stamp: ''!
sourceFiles
	^ OBSourceFilesResource current sources! !

!OBVersionsBrowserTest methodsFor: 'running' stamp: ''!
tearDown
	(BogusA compiledMethodAt: #two) setSourcePointer: 0! !

!OBVersionsBrowserTest methodsFor: 'testing' stamp: ''!
testInitialDisplay
	self assertListAt: 1 contains: self expectedHeadings! !

!OBVersionsBrowserTest methodsFor: 'testing' stamp: 'cwp 11/27/2004 21:39'!
testSourceDisplay
	self select: 'BogusA>>two cwp 3/11/2004 20:07'.
	self assertTextIs: self twoLatestVersionText! !

!OBVersionsBrowserTest methodsFor: 'constants' stamp: ''!
twoLatestVersionText
	^  'two
	^ 6' asText makeSelectorBold! !

!OBVersionsBrowserTest methodsFor: 'constants' stamp: ''!
twoSourcePointer
	^ 33555020! !

!ClassOrganizer methodsFor: '*ob-standard-testing' stamp: 'cwp 9/18/2004 23:59'!
isClassOrganizer
	^ true! !

Object subclass: #OBClassReference
	instanceVariableNames: 'name isMeta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBClassReference commentStamp: 'cwp 1/8/2005 13:15' prior: 0!
OBClassReference provides a way to refer to classes that may or may not be loaded into the image. It refers to the class indirectly, via name, rather than with a direct pointer to the class object. It also provides a number of convenience methods, which makes it more convenient than using the class name directly.!

!OBClassReference class methodsFor: 'instance creation' stamp: ''!
metaNamed: aSymbol
	^ (self named: aSymbol) beMeta! !

!OBClassReference class methodsFor: 'instance creation' stamp: ''!
named: aSymbol
	^ self new setName: aSymbol! !

!OBClassReference class methodsFor: 'instance creation' stamp: ''!
to: aClass
	^ self named: aClass name! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
beMeta
	isMeta := true! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
beNonMeta
	isMeta := false! !

!OBClassReference methodsFor: 'comparing' stamp: ''!
hash
	^ isMeta
		ifTrue: [name hash bitInvert]
		ifFalse: [name hash]! !

!OBClassReference methodsFor: 'testing' stamp: ''!
isMeta
	^ isMeta! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
name
	^ isMeta
		ifTrue: [name, ' class']
		ifFalse: [name]! !

!OBClassReference methodsFor: 'printing' stamp: ''!
printOn: aStream
	aStream nextPutAll: 'OBClassReference'.
	aStream nextPut: $<.
	aStream nextPutAll: name.
	isMeta ifTrue: [aStream nextPutAll: ' class'].
	aStream nextPut: $>.! !

!OBClassReference methodsFor: 'converting' stamp: ''!
referenceToMethod: aSelector
	| ref |
	ref := MethodReference new.
	ref setClassSymbol: name classIsMeta: isMeta  methodSymbol: aSelector stringVersion: ''.
	^ ref! !

!OBClassReference methodsFor: 'initializing' stamp: ''!
setName: aSymbol
	name := (aSymbol copyUpTo: $ ) asSymbol.
	isMeta := aSymbol endsWith: ' class'.! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theClass
	| theClass |
	theClass := self theNonMetaClass ifNil: [^ nil].
	^ isMeta
		ifFalse: [theClass]
		ifTrue: [theClass class]! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theNonMetaClass
	^ Smalltalk at: name ifAbsent: [].! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theNonMetaName
	^ name! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theNonMetaName: aSymbol
	name := aSymbol! !

!OBClassReference methodsFor: 'comparing' stamp: ''!
<= other
	^ self name <= other name! !

!OBClassReference methodsFor: 'comparing' stamp: ''!
= other
	^ (other isKindOf: self class) 
		and: [name = other theNonMetaName] 
		and: [isMeta = other isMeta]! !

Object subclass: #OBMethodVersion
	instanceVariableNames: 'sources pointer classRef className category stamp prior selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBMethodVersion commentStamp: 'cwp 1/8/2005 13:25' prior: 0!
OBMethodVerison represents a version of a method found in the .sources or .changes files. It provides methods for parsing the method definition referenced by a source pointer, and for filing in the source to replace the current version of the method.!

!OBMethodVersion class methodsFor: 'instance creation' stamp: ''!
fromSources: sources andPointer: pointer
	^ self new setSources: sources pointer: pointer! !

!OBMethodVersion class methodsFor: 'scanning' stamp: ''!
scan: sources from: pointer 
	| versions current |
	
	versions := OrderedCollection new.
	current := OBMethodVersion fromSources: sources andPointer: pointer.
	[current notNil]
		whileTrue: [versions add: current.
					current := current previous].
	^ versions! !

!OBMethodVersion class methodsFor: 'scanning' stamp: ''!
versionsOfMethod: methodReference 
	| class selector |
	class := methodReference actualClass.
	selector := methodReference methodSymbol.
	^ self scan: SourceFiles from: (class compiledMethodAt: selector) sourcePointer! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
category
	^ category! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
classSymbol
	^ classRef name! !

!OBMethodVersion methodsFor: 'compiling' stamp: ''!
fileIn
	(self theClass) ifNotNilDo: [:class | class
						 				compile: self source 
										classified: self category 
										withStamp: self stamp 
										notifying: nil]
						
						! !

!OBMethodVersion methodsFor: 'comparing' stamp: 'cwp 10/17/2004 23:45'!
hash
	^ pointer hash! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:51'!
latest
	^ self class 
			fromSources: sources 
			andPointer: (classRef theClass compiledMethodAt: selector) sourcePointer! !

!OBMethodVersion methodsFor: 'initializing' stamp: 'cwp 11/6/2004 23:14'!
parseChunk: aString
	| tokens |
	tokens := Scanner new scanTokens: aString.
	classRef := OBClassReference named: tokens first.
	tokens second = #class
		ifTrue: [classRef beMeta.
				category := tokens fourth.
				stamp := tokens sixth]
		ifFalse: [category := tokens third.
				tokens size > 3 ifTrue: [stamp := tokens fifth]].
	tokens size > 6 ifTrue: [prior := tokens last].! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
parseSource
	| file position chunk |
	file := sources at: (sources fileIndexFromSourcePointer: pointer).
	position := sources filePositionFromSourcePointer: pointer.
	position > file size ifTrue: [self error: 'Invalid source pointer'].
	
	file position: (0 max: position-150).  	"Skip back to before the preamble"
		[file position < (position-1)]  	"then pick it up from the front"
			whileTrue: [chunk := file nextChunk].
	
	self parseChunk: chunk.! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:42'!
pointer
	^ pointer! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
previous
	^ prior ifNotNil: [self class fromSources: sources andPointer: prior]! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
selector
	^ selector ifNil: [self setSelector]! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
setSelector
	| file |
	file := sources at: (sources fileIndexFromSourcePointer: pointer).
	file position: (sources filePositionFromSourcePointer: pointer).
	^ selector := Parser new parseSelector: file nextChunk.! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
setSources: aSourceFileArray pointer: aSourcePointer
	sources := aSourceFileArray.
	pointer := aSourcePointer.
	self parseSource.! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
source
	| file |
	file := sources at: (sources fileIndexFromSourcePointer: pointer).
	file position: (sources filePositionFromSourcePointer: pointer).
	^ file nextChunk asText makeSelectorBold! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
stamp
	^ stamp! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
theClass
	^ classRef theClass! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
theClassName
	^ classRef name! !

!OBMethodVersion methodsFor: 'comparing' stamp: 'cwp 10/17/2004 23:45'!
= other
	^ self species = other species and: [self pointer = other pointer]! !

Object subclass: #OBSystemBrowserAdaptor
	instanceVariableNames: 'class selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBSystemBrowserAdaptor commentStamp: 'cwp 1/8/2005 13:35' prior: 0!
OBSystemBrowserAdaptor implements the protocol expected by the SystemBrowser app registry, and thus allows OmniBrowser to be chosen as the default system browser. 

Caveat: because the required protocol is a little ...odd.... it can't be implemented well by OBSystemBrowser directly. OBSystemBrowserAdaptor does this reasonably well, but it has no way to provide a more natural name than its class name to the app registry menu.!

!OBSystemBrowserAdaptor class methodsFor: 'initializing' stamp: 'cwp 12/5/2004 22:01'!
initialize
	self register! !

!OBSystemBrowserAdaptor class methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:59'!
openBrowser
	OBSystemBrowser open! !

!OBSystemBrowserAdaptor class methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:56'!
openBrowserView: adaptor label: ignored 
	^ OBSystemBrowser 
		openOnClass: adaptor targetClass 
		selector: adaptor selector! !

!OBSystemBrowserAdaptor class methodsFor: 'initializing' stamp: 'cwp 1/8/2005 21:52'!
register
	Smalltalk at: #SystemBrowser ifPresent: [:class | class register: self]! !

!OBSystemBrowserAdaptor class methodsFor: 'initializing' stamp: 'cwp 1/8/2005 21:53'!
unregister
	Smalltalk at: #SystemBrowser ifPresent: [:class | class unregister: self]! !

!OBSystemBrowserAdaptor methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:50'!
labelString
	^ 'System Browser'! !

!OBSystemBrowserAdaptor methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:49'!
openEditString: anUndefinedObject 
	^ self! !

!OBSystemBrowserAdaptor methodsFor: 'accessing' stamp: 'cwp 12/5/2004 21:55'!
selector
	^ selector! !

!OBSystemBrowserAdaptor methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:49'!
setClass: aClass selector: aSelector 
	class _ aClass.
	selector _ aSelector! !

!OBSystemBrowserAdaptor methodsFor: 'accessing' stamp: 'cwp 12/5/2004 21:55'!
targetClass
	^ class! !

!OBDefinitionPanel methodsFor: '*ob-standard-cmds' stamp: 'cwp 12/5/2004 18:21'!
browseIt: aSymbol
	| entry |
	entry := self selectedClass environment at: aSymbol ifAbsent: [nil].
	entry ifNil: [^ self implementorsOfIt: aSymbol].
	(entry isKindOf: Class) ifFalse: [entry _ entry class].
	OBSystemBrowser openOnClass: entry.
	^ true
	! !

!OBDefinitionPanel methodsFor: '*ob-standard-cmds' stamp: 'avi 9/17/2005 01:36'!
implementorsOfIt: aSelector 
	OBImplementorsBrowser browseRoot: (OBSelectorNode on: aSelector).
	^true! !

!OBDefinitionPanel methodsFor: '*ob-standard-cmds' stamp: 'avi 9/17/2005 01:36'!
referencesToIt: aClassName 
	| class |
	class := self selectedClass environment at: aClassName ifAbsent: [^false].
	class isBehavior ifFalse: [^false].
	OBReferencesBrowser browseRoot: (OBClassNode on: class).
	^true! !

!OBDefinitionPanel methodsFor: '*ob-standard-cmds' stamp: 'avi 9/17/2005 01:36'!
sendersOfIt: aSelector 
	OBSendersBrowser browseRoot: (OBSelectorNode on: aSelector).
	^true! !

OBOpenTest subclass: #OBStandardOpenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBStandardOpenTest methodsFor: 'tests' stamp: 'avi 9/17/2005 01:29'!
testListBrowseRoot
	self open: [OBSendersBrowser browseRoot: (OBMethodNode on: #one inClass: BogusA)].
	self assert: browser navigationPanel minPanes = 1.
	self assert: browser navigationPanel maxPanes = 2.
	self assert: browser currentNode isNil! !

!OBStandardOpenTest methodsFor: 'tests' stamp: 'avi 9/17/2005 01:37'!
testListBrowseRootTitle
	self open: [OBSendersBrowser 
				browseRoot: (OBMethodNode on: #one inClass: BogusA)
				title: 'My senders of'].
	self assert: browser navigationPanel minPanes = 1.
	self assert: browser navigationPanel maxPanes = 2.
	self assert: browser currentNode isNil.
	self assert: browser defaultLabel = 'My senders of #one'! !

!OBStandardOpenTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 17:44'!
testSystemOpen
	self open: [OBSystemBrowser open].
	self assert: browser navigationPanel minPanes = 4.
	self assert: browser navigationPanel maxPanes = 4.
	self assert: browser currentNode = nil! !

!OBStandardOpenTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 18:31'!
testSystemOpenOnClass
	self open: [OBSystemBrowser openOnClass: BogusA].
	self assert: browser navigationPanel minPanes = 4.
	self assert: browser navigationPanel maxPanes = 4.
	self assert: browser currentNode name = #BogusA! !

!OBStandardOpenTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 17:44'!
testSystemOpenOnClassSelector
	self open: [OBSystemBrowser openOnClass: BogusA selector: #one].
	self assert: browser navigationPanel minPanes = 4.
	self assert: browser navigationPanel maxPanes = 4.
	self assert: browser currentNode name = #one! !

!OBStandardOpenTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 17:52'!
testVersionsOpenOn
	| root |
	root _ OBMethodNode on: #testVersionsOpenOn inClass: self class.
	self open: [OBVersionBrowser openOn: root].
	self assert: browser navigationPanel minPanes = 1.
	self assert: browser navigationPanel maxPanes = 1.
	self assert: browser currentNode isNil.
	self assert: browser root = root! !

TestResource subclass: #OBBrowserResource
	instanceVariableNames: 'model morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBBrowserResource methodsFor: 'accessing' stamp: ''!
model
	^ model! !

!OBBrowserResource methodsFor: 'accessing' stamp: ''!
morph
	^ morph! !

!OBBrowserResource methodsFor: 'running' stamp: 'cwp 12/5/2004 18:13'!
setUp
	model := OBBrowser metaNode: self metagraph node: self node.
	model addPanel: (OBDefinitionPanel inBrowser: model).
	morph := model morph.! !

OBBrowserResource subclass: #OBImageBrowserResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBImageBrowserResource methodsFor: 'running' stamp: 'avi 9/17/2005 01:13'!
metagraph
	^ OBSystemBrowser alphabeticalMetaNode! !

!OBImageBrowserResource methodsFor: 'running' stamp: ''!
node
	^ OBEnvironmentNode forImage! !

TestResource subclass: #OBSourceFilesResource
	instanceVariableNames: 'sources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBSourceFilesResource methodsFor: 'initializing' stamp: ''!
changesStream
	^ ReadStream on: self changesString! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: ''!
changesString
	^ '!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:05'' prior: 16777764!!
one
	^ 4!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 16777864!!
two
	^ 4!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 33554520!!
one
	^ 5!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 33554620!!
two
	^ 5!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 33554720!!
one
	^ 6!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 33554820!!
two
	^ 6!! !!
!!BogusA class methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 16777462!!
three
	^ 2!! !!'! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: ''!
setUp
	sources := StandardSourceFileArray new: 2.
	sources at: 1 put: self sourceStream.
	sources at: 2 put: self changesStream.! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: ''!
sourceStream
	^ ReadStream on: self sourceString! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: ''!
sourceString
	^ '!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:05''!!
one
	^ 1!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06''!!
two
	^ 1!! !!
!!BogusA class methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06''!!
three
	^ 1!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 16777288!!
one
	^ 2!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 16777372!!
two
	^ 2!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 16777600!!
one
	^ 3!! !!
!!BogusA methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 16777664!!
two
	^ 3!! !!'! !

!OBSourceFilesResource methodsFor: 'accessing' stamp: ''!
sources
	^ sources! !

TestCase subclass: #OBAncestryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBAncestryTest methodsFor: 'asserting' stamp: ''!
assert: ancestor isAncestorOf: descendant
	self assert: (ancestor isAncestorOf: descendant)! !

!OBAncestryTest methodsFor: 'asserting' stamp: ''!
deny: ancestor isAncestorOf: descendant
	self deny: (ancestor isAncestorOf: descendant)! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testClassCategoryIdentity
	self 
		assert: (OBClassCategoryNode on: 'Bogus-Base' inEnvironment: Smalltalk) 
		isAncestorOf: (OBClassCategoryNode on: 'Bogus-Base' inEnvironment: Smalltalk).
! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testClassCategoryOther
	self 
		deny: (OBClassCategoryNode on: 'Bogus-Base' inEnvironment: Smalltalk)
		isAncestorOf: (OBClassCategoryNode on: 'Bogus-Formats' inEnvironment: Smalltalk)
	! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testClassIdentity
	self 
		assert: (OBClassNode on: BogusA)
		isAncestorOf: (OBClassNode on: BogusA)! !

!OBAncestryTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 21:46'!
testClassInClassCategory
	| cat |
	cat := OBClassCategoryNode on: 'Bogus-Base' inEnvironment: Smalltalk.
	self assert: cat isAncestorOf: (OBClassNode on: BogusA).
	
! !

!OBAncestryTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 21:46'!
testClassInOtherCategory
	| cat |
	cat := OBClassCategoryNode on: 'Bogus-Base' inEnvironment: Smalltalk.
	self deny: cat isAncestorOf: (OBClassNode on: BogusF)
	
! !

!OBAncestryTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 12:55'!
testClassMethodInClassCat
	self 
		assert: (OBClassCategoryNode on: #'Bogus-Base' inEnvironment: Smalltalk)
		isAncestorOf: (OBMethodNode on: #a inClass: BogusA class)
! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testClassOther
	self 
		deny: (OBClassNode on: BogusB)
		isAncestorOf: (OBClassNode on: BogusA)! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testMethodCatIdentity
	self 
		assert: (OBMethodCategoryNode on: #letters inClass: BogusA)
		isAncestorOf: (OBMethodCategoryNode on: #letters inClass: BogusA)! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testMethodCatOther
	self 
		deny: (OBMethodCategoryNode on: #numbers inClass: BogusA)
		isAncestorOf: (OBMethodCategoryNode on: #letters inClass: BogusA)! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testMethodIdentity
	self 
		assert: (OBMethodNode on: #a inClass: BogusA)
		isAncestorOf: (OBMethodNode on: #a inClass: BogusA)! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testMethodInClass
	self assert: (OBClassNode on: BogusA) isAncestorOf: (OBMethodNode on: #a inClass: BogusA).
! !

!OBAncestryTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 12:54'!
testMethodInClassCat
	self 
		assert: (OBClassCategoryNode on: #'Bogus-Base' inEnvironment: Smalltalk)
		isAncestorOf: (OBMethodNode on: #a inClass: BogusA)
! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testMethodInMetaclass
	self deny: (OBClassNode on: BogusA) isAncestorOf: (OBMethodNode on: #a inClass: BogusA class)
	! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testMethodInMethodCat
	self 
		assert: (OBMethodCategoryNode on: #letters inClass: BogusA)
		isAncestorOf: (OBMethodNode on: #a inClass: BogusA)! !

!OBAncestryTest methodsFor: 'tests' stamp: ''!
testMethodInOtherClass
	self 
		deny: (OBClassNode on: BogusA)
		isAncestorOf: (OBMethodNode on: #isString inClass: BogusE).
	! !

!OBAncestryTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 21:48'!
testMethodOverride
	self 
		assert: (OBMethodNode on: #isString inClass: BogusA)
		isAncestorOf: (OBMethodNode on: #isString inClass: BogusE).
	! !

!OBAncestryTest methodsFor: 'tests' stamp: 'cwp 12/8/2004 21:49'!
testMethodSiblings
	self 
		deny: (OBMethodNode on: #isString inClass: AbstractString)
		isAncestorOf: (OBMethodNode on: #isString inClass: BogusA).
	! !

TestCase subclass: #OBClassHierarchyTest
	instanceVariableNames: 'classes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBClassHierarchyTest methodsFor: 'constants' stamp: 'cwp 12/11/2004 00:13'!
bogusA
	^ self nodeForClassNamed: #BogusA! !

!OBClassHierarchyTest methodsFor: 'constants' stamp: 'cwp 12/11/2004 00:13'!
bogusB
	^ self nodeForClassNamed: #BogusB! !

!OBClassHierarchyTest methodsFor: 'constants' stamp: 'cwp 12/11/2004 00:29'!
bogusE
	^ self nodeForClassNamed: #BogusE! !

!OBClassHierarchyTest methodsFor: 'constants' stamp: 'cwp 12/11/2004 00:12'!
nodeForClassNamed: aSymbol
	^ classes detect: [:ea | ea theClassName = aSymbol]! !

!OBClassHierarchyTest methodsFor: 'running' stamp: 'cwp 12/12/2004 22:24'!
setUp
	| metacat cat |
	metacat := OBMetaNode new childAt: #classesHierarchically
				put: (OBMetaNode new displaySelector: #name).
	cat := OBClassCategoryNode on: BogusInfo baseCategory
				inEnvironment: BogusInfo environment.
	classes := metacat nodesForParent: cat! !

!OBClassHierarchyTest methodsFor: 'tests' stamp: 'cwp 12/11/2004 00:11'!
testAB
	self assert: (classes indexOf: self bogusA) < (classes indexOf: self bogusB)! !

!OBClassHierarchyTest methodsFor: 'tests' stamp: 'cwp 12/11/2004 00:33'!
testAE
	self assert: (classes indexOf: self bogusA) < (classes indexOf: self bogusE)! !

!OBClassHierarchyTest methodsFor: 'tests' stamp: 'cwp 12/11/2004 00:29'!
testEB
	self assert: (classes indexOf: self bogusE) < (classes indexOf: self bogusB)! !

TestCase subclass: #OBCodeBrowserUiTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBCodeBrowserUiTest methodsFor: 'running' stamp: 'cwp 12/6/2004 22:11'!
setUp
	browser _ OBSystemBrowser new.
	browser addDependent: self.
	! !

!OBCodeBrowserUiTest methodsFor: 'running' stamp: 'cwp 12/6/2004 22:11'!
tearDown
	browser removeDependent: self! !

!OBCodeBrowserUiTest methodsFor: 'tests' stamp: 'cwp 12/6/2004 22:20'!
testWindowMenu
	| menu |
	menu _ MenuMorph new.
	browser addModelItemsToWindowMenu: menu.
	self deny: (menu itemWithWording: 'Register this Browser as default') isNil.
	self deny: (menu itemWithWording: 'Choose new default browser') isNil
		! !

TestCase subclass: #OBCodeNodeEqualityTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:47'!
testClassCategoryEquality
	self assert: (OBClassCategoryNode on: #'Bogus-Base' inEnvironment: Smalltalk)
					=  (OBClassCategoryNode on: #'Bogus-Base' inEnvironment: Smalltalk)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:48'!
testClassCategoryInequality
	self deny: (OBClassCategoryNode on: #'Bogus-Base' inEnvironment: Smalltalk)
					=  (OBClassCategoryNode on: #'Bogus-Formats' inEnvironment: Smalltalk)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 22:21'!
testClassCommentEquality
	self assert: (OBClassCommentNode on: BogusA)
					= (OBClassCommentNode on: BogusA)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 22:23'!
testClassCommentInequality
	self deny: (OBClassCommentNode on: BogusA)
					= (OBClassCommentNode on: BogusB)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:44'!
testClassEquality
	self assert: (OBClassNode on: BogusA)
					= (OBClassNode on: BogusA)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:45'!
testClassInequality
	self deny: (OBClassNode on: BogusA)
					= (OBClassNode on: BogusB)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:39'!
testMethod
	self assert: (OBMethodNode on: #one inClass: BogusA)
					= (OBMethodNode on: #one inClass: BogusA)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:46'!
testMethodCategoryEquality
	self assert: (OBMethodCategoryNode on: #numbers inClass: BogusA)
					= (OBMethodCategoryNode on: #numbers inClass: BogusA)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:46'!
testMethodCategoryInequality1
	self deny: (OBMethodCategoryNode on: #numbers inClass: BogusA)
					= (OBMethodCategoryNode on: #letters inClass: BogusA)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:47'!
testMethodCategoryInequality2
	self deny: (OBMethodCategoryNode on: #numbers inClass: BogusA)
					= (OBMethodCategoryNode on: #numbers inClass: BogusB)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:45'!
testMethodEquality
	self assert: (OBMethodNode on: #one inClass: BogusA)
					= (OBMethodNode on: #one inClass: BogusA)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:45'!
testMethodInequality1
	self deny: (OBMethodNode on: #one inClass: BogusA)
					= (OBMethodNode on: #two inClass: BogusA)! !

!OBCodeNodeEqualityTest methodsFor: 'tests' stamp: 'cwp 12/20/2004 00:46'!
testMethodInequality2
	self deny: (OBMethodNode on: #one inClass: BogusA)
					= (OBMethodNode on: #one inClass: BogusB)! !

TestCase subclass: #OBCodeSanityCheckTest
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBCodeSanityCheckTest methodsFor: 'constants' stamp: ''!
bogusARedefinition
	^ BogusA definition! !

!OBCodeSanityCheckTest methodsFor: 'constants' stamp: ''!
bogusCDefinition
	^ 'Object subclass: #BogusC
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', BogusInfo baseCategory, ''''! !

!OBCodeSanityCheckTest methodsFor: 'constants' stamp: ''!
codeContext
	^ OBImageContext new! !

!OBCodeSanityCheckTest methodsFor: 'callbacks' stamp: ''!
selectionInterval
	^ 1 to: 0! !

!OBCodeSanityCheckTest methodsFor: 'testing' stamp: ''!
testClassCreation
	node := OBClassCategoryNode on: BogusInfo baseCategory inEnvironment: Smalltalk.
	self 
		should: [node definition accept: self bogusCDefinition notifying: self]
		raise: OBConfirmationRequest! !

!OBCodeSanityCheckTest methodsFor: 'testing' stamp: ''!
testClassDefinition
	node := OBClassNode on: BogusA.
	self 
		should: [node definition accept: self bogusCDefinition notifying: self]
		raise: OBConfirmationRequest! !

!OBCodeSanityCheckTest methodsFor: 'testing' stamp: ''!
testClassRedefinition
	node := OBClassNode on: BogusA.
	self 
		shouldnt: [node accept: self bogusARedefinition notifying: self]
		raise: OBConfirmationRequest! !

!OBCodeSanityCheckTest methodsFor: 'callbacks' stamp: ''!
text
	^ node text asText! !

TestCase subclass: #OBMethodVersionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBMethodVersionTest methodsFor: 'constants' stamp: ''!
fakeSourceFileArray
	^ OBSourceFilesResource current sources! !

!OBMethodVersionTest methodsFor: 'constants' stamp: ''!
methodReference
	| ref |
	ref := MethodReference new.
	ref setStandardClass: self class methodSymbol: #testScanFromMethodRef.
	^ ref! !

!OBMethodVersionTest methodsFor: 'testing' stamp: ''!
testScanFromMethodRef
	| changes |
	changes := OBMethodVersion versionsOfMethod: self methodReference.
	self deny: changes isEmpty! !

!OBMethodVersionTest methodsFor: 'testing' stamp: ''!
testScanFromPointer
	| changes sourcePointer sources |
	sources :=  self fakeSourceFileArray.
	sourcePointer := sources sourcePointerFromFileIndex: 2 andPosition: 488.
	changes := OBMethodVersion scan: sources from: sourcePointer.
	self assert: changes size = 6.
	changes do: [:ea | 
					self assert: ea theClassName = #BogusA.
					self assert: ea category = 'as yet unclassified'.
					self assert: (ea stamp beginsWith: 'cwp 3/11/2004 20:0')]! !

!OBMethodVersionTest methodsFor: 'testing' stamp: ''!
testScanFromPointerToClassMethod
	| changes sourcePointer sources |
	sources :=  self fakeSourceFileArray.
	sourcePointer := sources sourcePointerFromFileIndex: 2 andPosition: 694.
	changes := OBMethodVersion scan: sources from: sourcePointer.
	self assert: changes size = 2.
	changes do: [:ea | 
					self assert: ea theClassName = #'BogusA class'.
					self assert: ea category = 'as yet unclassified'.
					self assert: (ea stamp beginsWith: 'cwp 3/11/2004 20:0')]! !

TestCase subclass: #OBSystemBrowserAdaptorTest
	instanceVariableNames: 'oldDefault browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBSystemBrowserAdaptorTest methodsFor: 'running' stamp: 'cwp 12/5/2004 21:58'!
openWith: aBlock
	aBlock
		on: OBBrowseRequest
		do: [:n | browser _ n browser].
	self deny: browser isNil.
! !

!OBSystemBrowserAdaptorTest methodsFor: 'running' stamp: 'cwp 12/5/2004 21:48'!
setUp
	oldDefault _ SystemBrowser defaultOrNil.
	SystemBrowser default: OBSystemBrowserAdaptor! !

!OBSystemBrowserAdaptorTest methodsFor: 'running' stamp: 'cwp 12/5/2004 21:43'!
tearDown
	SystemBrowser default: oldDefault! !

!OBSystemBrowserAdaptorTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 21:59'!
testDefaultOpenBrowser
	self openWith: [SystemBrowser defaultOpenBrowser].
	self assert: browser currentNode isNil! !

!OBSystemBrowserAdaptorTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 21:58'!
testFullOnClassSelector
	self openWith: [Browser fullOnClass: BogusA selector: #one].
	self assert: browser currentNode theClass = BogusA.
	self assert: browser currentNode selector = #one! !

OBDefinition subclass: #OBClassDefinition
	instanceVariableNames: 'environment template'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Definitions'!

!OBClassDefinition commentStamp: 'cwp 1/8/2005 13:13' prior: 0!
OBClassDefinition presents a textual interface for examining, modifying and creating classes. Given a class, it knows how to display the definition expression that reflects it's current state, and knows how to create or modify a class given a definition expression.

OBClassDefinition implements a number of safety checks when defining or redefining classes, to ensure that existing classes are not accidentally overwritten.!

!OBClassDefinition class methodsFor: 'instance creation' stamp: ''!
environment: anEnvironment template: aString 
	^ self new setEnvironment: anEnvironment template: aString! !

!OBClassDefinition methodsFor: 'callbacks' stamp: ''!
accept: aText notifying: aController
	^ self defineClass: aText notifying: aController! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
confirmDefinition: definition
	"Check to make sure the user isn't accidentally over-writing an existing class."
	
	(((self isRedefinition: definition) not) and: [self definedClassExists: definition])
		ifTrue: [^ self confirmRedefinition: definition]
		ifFalse: [^ true]! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
confirmRedefinition: definition
	| newName prompt |
	newName := self nameOfClassDefinedBy: definition.
	prompt := (newName, ' is an existing class in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText.
	^ OBConfirmationRequest
		prompt: prompt
		confirm: 'Redefine'! !

!OBClassDefinition methodsFor: 'class definition' stamp: ''!
defineClass: definition notifying: aController
	| evaluator newClass |
	(self confirmDefinition: definition) ifFalse: [^ false].
	evaluator := self evaluatorForDefinition: definition.
	newClass := evaluator
				evaluate: definition
				notifying: aController
				logged: true.
	newClass ifNil: [^ false].
	self signalSelectionOf: newClass.
	^ true
! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
definedClassExists: definition
	^ environment hasClassNamed: (self nameOfClassDefinedBy: definition)! !

!OBClassDefinition methodsFor: 'class definition' stamp: ''!
evaluatorForDefinition: definition
	| tokens |
	tokens := Scanner new scanTokens: definition.
	^ (environment at: tokens first ifAbsent: [nil]) subclassDefinerClass! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
isRedefinition: aDefinition
	^ (self nameOfClassDefinedBy: aDefinition) = (self nameOfClassDefinedBy: template)! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
nameOfClassDefinedBy: definition
	^ (Scanner new scanTokens: definition) third! !

!OBClassDefinition methodsFor: 'confirmation' stamp: 'cwp 10/18/2004 00:13'!
selectedClass
	^ environment at: (self nameOfClassDefinedBy: template) ifAbsent: [nil]! !

!OBClassDefinition methodsFor: 'callbacks' stamp: ''!
selection
	^ 1 to: 0! !

!OBClassDefinition methodsFor: 'initializing' stamp: ''!
setEnvironment: anEnvironment template: aText
	environment := anEnvironment.
	template := aText.! !

!OBClassDefinition methodsFor: 'class definition' stamp: ''!
signalSelectionOf: aClass
	((aClass isMeta
		ifTrue: [OBMetaclassNode] 
		ifFalse: [OBClassNode]) 
			on: aClass) signalSelection! !

!OBClassDefinition methodsFor: 'callbacks' stamp: ''!
text
	^ template! !

OBDefinition subclass: #OBMethodDefinition
	instanceVariableNames: 'theClass category source selection callback'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Definitions'!

!OBMethodDefinition commentStamp: 'cwp 1/8/2005 13:20' prior: 0!
OBMethodDefinition knows how to present method source in the browser's text pane and compile a new CompiledMethod when the source changes. It implements several safety checks to ensure that methods are not accidentally overwritten.!

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:07'!
inCategory: aString inClass: aClass
	^ self 
		selection: nil
		source: nil
		inCategory: aString
		inClass: aClass! !

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:07'!
selection: anInterval source: aText inCategory: aCategory inClass: aClass
	^ self new
		setClass: aClass
		category: aCategory
		source: aText
		selection: anInterval! !

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:07'!
selection: anInterval source: aText inClass: aClass
	^ self 
		selection: anInterval
		source: aText
		inCategory: nil
		inClass: aClass	! !

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:03'!
source: aText inClass: aClass
	^ self 
		selection: nil
		source: aText
		inCategory: nil
		inClass: aClass! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: 'cwp 12/2/2004 23:29'!
accept: aText notifying: aController
	| newSelector |
	newSelector := self compileMethod: aText notifying: aController.
	newSelector ifNil: [^ false].
	(self nodeFor: newSelector) signalSelection.
	^ true! !

!OBMethodDefinition methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:36'!
callback: aBlockContext 
	callback _ aBlockContext! !

!OBMethodDefinition methodsFor: 'accessing' stamp: ''!
category
	^ category ifNil: [category := self theClass whichCategoryIncludesSelector: self selector]! !

!OBMethodDefinition methodsFor: 'compiling' stamp: ''!
compileMethod: aText notifying: aController
	^ (self confirmMethod: aText)
		ifTrue: [self theClass 
					compile: aText 
					classified: self category 
					notifying: aController]
		ifFalse: [nil]! !

!OBMethodDefinition methodsFor: 'confirming' stamp: 'cwp 10/17/2004 22:42'!
confirmMethod: aText
	| sel |
	sel _ Parser new parseSelector: aText.
	^ (self theClass isMeta 
			and: [(self selectorAlreadyDefined: sel) not] 
			and: [Metaclass isScarySelector: sel])
				ifTrue: [self confirmScarySelector: sel]
				ifFalse: [^ true]! !

!OBMethodDefinition methodsFor: 'confirming' stamp: ''!
confirmScarySelector: aSelector
	| prompt |
	prompt := aSelector, ' is used in the existing class system.
Overriding it could cause serious problems.
Is this really what you want to do?'.

	^ OBConfirmationRequest
		prompt: prompt
		confirm: 'Override'! !

!OBMethodDefinition methodsFor: 'compiling' stamp: 'cwp 10/17/2004 23:33'!
nodeFor: aSelector
	^ callback value: aSelector! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: 'cwp 10/17/2004 23:16'!
redefineSource: aText selector: newSelector
	newSelector = self selector 
		ifTrue: [source := aText]
		ifFalse: [(self methodNodeFor: newSelector) signalSelection].
! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: ''!
selectedClass
	^ self theClass! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: ''!
selection
	^ selection ifNil: [1 to: (source ifNil: [self text size] ifNotNil: [0])]! !

!OBMethodDefinition methodsFor: 'accessing' stamp: ''!
selector
	^ source ifNotNil: [Parser new parseSelector: source]! !

!OBMethodDefinition methodsFor: 'confirming' stamp: ''!
selectorAlreadyDefined: aSelector
	^ self theClass selectors includes: aSelector! !

!OBMethodDefinition methodsFor: 'initializing' stamp: 'cwp 10/17/2004 23:33'!
setClass: aClass category: aString source: aText selection: anInterval
	theClass := aClass.
	category := aString.
	source := aText.
	selection := anInterval.
	callback := [:sel | OBMethodNode on: sel inClass: theClass]! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: 'cwp 11/29/2004 21:21'!
text
	^ source 
		ifNotNil: [source asText makeSelectorBold]
		ifNil: [self theClass sourceCodeTemplate]! !

!OBMethodDefinition methodsFor: 'accessing' stamp: ''!
theClass
	^ theClass! !

OBDefinition subclass: #OBOrganizationDefinition
	instanceVariableNames: 'organizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Definitions'!

!OBOrganizationDefinition commentStamp: 'cwp 1/8/2005 13:31' prior: 0!
OBOrganizationDefinition knows how to display and edit the organization of classes or methods by a Categorizer.!

!OBOrganizationDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 9/22/2004 21:20'!
on: anOrganizer
	^ self new organizer: anOrganizer! !

!OBOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 9/22/2004 21:21'!
organizer: anOrganizer
	organizer _ anOrganizer! !

!OBOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 9/22/2004 21:18'!
text
	^ organizer asString! !

!OBOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 1/9/2005 11:45'!
text: aText
	organizer changeFromString: aText asString.
	^ true! !

!Class methodsFor: '*ob-standard-converting' stamp: ''!
asNode
	^ OBClassNode on: self! !

OBBrowser subclass: #OBCodeBrowser
	instanceVariableNames: 'hasChanges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBCodeBrowser commentStamp: 'cwp 1/7/2005 23:45' prior: 0!
OBCodeBrowser is a superclass for all browsers which active code in the image. It provides methods for registering with the SystemChangeNotifier and updating it's display when it receives notification of system changes.!

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:07'!
addTo: root class: classSel comment: commentSel metaclass: metaclassSel 
	| class metaclass comment methodCategory method allMethodCategory |
	class := OBMetaNode named: 'Class'.
	comment := OBMetaNode named: 'ClassComment'.
	metaclass := OBMetaNode named: 'Metaclass'.
	allMethodCategory := OBMetaNode named: 'AllMethodCategory'.
	methodCategory := OBMetaNode named: 'MethodCategory'.
	method := OBMetaNode named: 'Method'.
	root
		childAt: classSel
			labeled: 'instance'
			put: class;
		childAt: commentSel
			labeled: '?'
			put: comment;
		childAt: metaclassSel
			labeled: 'class'
			put: metaclass;
		addActor: (OBNodeActor onNodeClass: OBClassCategoryNode);
		addActor: OBCategoryActor new;
		filterClass: OBModalFilter.
	class
		displaySelector: #indentedName;
		childAt: #allCategory put: allMethodCategory;
		childAt: #categories put: methodCategory;
		addActor: OBNodeActor new;
		addActor: OBClassActor new.
	comment 
		displaySelector: #indentedName;
		addActor: OBNodeActor new.
	metaclass
		displaySelector: #indentedName;
		childAt: #categories put: methodCategory;
		addActor: OBNodeActor new.
	allMethodCategory
		childAt: #methods put: method;
		addActor: (OBNodeActor onNodeClass: OBMethodCategoryNode).
	methodCategory
		childAt: #methods put: method;
		addActor: OBCategoryActor new;
		addActor: (OBNodeActor onNodeClass: OBMethodCategoryNode).
	method addActor: OBNodeActor new.
	^ root! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'cwp 12/15/2004 22:52'!
optionalButtons
	^ #('browse' 'variables' 'hierarchy' 'inheritance' 'senders' 'implementors' 'versions')! !

!OBCodeBrowser methodsFor: 'morphic' stamp: 'cwp 1/8/2005 21:55'!
addModelItemsToWindowMenu: aMenu
	Smalltalk 
		at: #SystemBrowser 
		ifPresent: [:class | class 
								addRegistryMenuItemsTo: aMenu 
								inAccountOf: OBSystemBrowserAdaptor new].! !

!OBCodeBrowser methodsFor: 'updating' stamp: 'cwp 1/7/2005 23:47'!
clearChanges
	hasChanges := false! !

!OBCodeBrowser methodsFor: 'updating' stamp: ''!
event: anEvent
	hasChanges := true! !

!OBCodeBrowser methodsFor: 'morphic' stamp: ''!
initialExtent
	hasChanges := false.
	self register.
	^ super initialExtent! !

!OBCodeBrowser methodsFor: 'updating' stamp: ''!
register
	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:! !

!OBCodeBrowser methodsFor: 'morphic' stamp: 'cwp 1/7/2005 23:47'!
stepAt: milliseconds in: aSystemWindow
	hasChanges ifTrue: [self refresh].
	self clearChanges! !

!OBCodeBrowser methodsFor: 'updating' stamp: ''!
unregister
	SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self! !

!OBCodeBrowser methodsFor: 'morphic' stamp: ''!
wantsStepsIn: aSystemWindow
	^ true! !

!OBCodeBrowser methodsFor: 'morphic' stamp: ''!
windowIsClosing
	self unregister! !

OBCodeBrowser subclass: #OBHierarchyBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBHierarchyBrowser commentStamp: 'cwp 1/7/2005 23:50' prior: 0!
OBHierarchyBrowser provides a three-pane browers that displays a class within it's surrounding hierarchy - both superclasses and subclasses!

!OBHierarchyBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:06'!
defaultMetaNode
	^ self 
		addTo: (OBMetaNode named: 'RootClass')
		class: #classHierarchy 
		comment: #commentHierarchy 
		metaclass: #metaclassHierarchy.
! !

!OBHierarchyBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/15/2004 22:49'!
onClass: aClass
	^ self root: (OBClassNode on: aClass)! !

!OBHierarchyBrowser class methodsFor: 'opening' stamp: 'cwp 12/15/2004 22:49'!
openOnClass: aClass
	^ (self onClass: aClass) open! !

!OBHierarchyBrowser class methodsFor: 'configuration' stamp: 'cwp 12/13/2004 00:37'!
paneCount
	^ 3! !

!OBHierarchyBrowser class methodsFor: 'configuration' stamp: 'cwp 12/13/2004 01:00'!
titleForRoot: aNode
	^ aNode name, ' Hierarchy'! !

!OBHierarchyBrowser methodsFor: 'as yet unclassified' stamp: 'cwp 12/13/2004 00:40'!
defaultBackgroundColor
	^ Color lightGreen! !

!OBHierarchyBrowser methodsFor: 'as yet unclassified' stamp: 'cwp 12/13/2004 01:00'!
defaultLabel
	^ self root name, ' Hierarchy'! !

OBCodeBrowser subclass: #OBInheritanceBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBInheritanceBrowser commentStamp: 'cwp 1/7/2005 23:51' prior: 0!
OBInheritanceBrowser shows the inheritance hierarchy of a method, both superclass implementations which it overrides, and subclass implementations which override it.!

!OBInheritanceBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:08'!
defaultMetaNode
	| method root |
	method := OBMetaNode named: 'Method'.
	root := OBMetaNode named: 'Root'.
	root
		childAt: #children put: method.
	method
		displaySelector: #fullName;
		childAt: #overrides put: method;
		addActor: OBNodeActor new.
	^ root! !

!OBInheritanceBrowser class methodsFor: 'configuration' stamp: ''!
title
	^ 'Inheritance'! !

!OBInheritanceBrowser class methodsFor: 'configuration' stamp: 'cwp 11/25/2004 22:05'!
titleForRoot: aCollectionNode
	^ 'Inheritance of ', aCollectionNode children first selector printString! !

!OBInheritanceBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:52'!
defaultBackgroundColor
	^ Color lightGreen! !

OBCodeBrowser subclass: #OBListBrowser
	instanceVariableNames: 'labelPrefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBListBrowser commentStamp: 'cwp 1/7/2005 23:53' prior: 0!
OBListBrowsers are used to display simple lists of methods, such as senders or implementors.!

OBListBrowser subclass: #OBImplementorsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBImplementorsBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:48'!
chaseImplementorsNav: navSelector
	| send message implementor |
	send := OBMetaNode named: 'Send'.
	message := OBMetaNode named: 'Message'.
	implementor := OBMetaNode named: 'Implementor'.
	implementor
		displaySelector: #indentedFullName;
		childAt: #messages put: message;
		childAt: #senders put: send;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	message
		childAt: navSelector labeled: 'implementors' put: implementor;
		childAt: #sendersOfMessage labeled: 'senders' put: send;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	send
		displaySelector: #fullName;
		childAt: navSelector put: implementor;
		childAt: #messages put: message;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	^ implementor! !

!OBImplementorsBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:24'!
defaultMetaNode
	^ self hierarchicalImplementors! !

!OBImplementorsBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:24'!
hierarchicalImplementors
	^ self implementorsNav: #implementorsHierarchically! !

!OBImplementorsBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:24'!
implementorsNav: navSelector
	| selector implementors |
	selector := OBMetaNode named: 'Selector'.
	implementors := OBMetaNode named: 'Implementors'.
	
	selector 
		childAt: navSelector labeled: 'list' put: implementors;
		childAt: navSelector labeled: 'chase' put: (self chaseImplementorsNav: navSelector);
		filterClass: OBModalFilter.
	implementors 
		displaySelector: #indentedFullName;
		addActor: OBNodeActor new.

	^ selector! !

!OBImplementorsBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:36'!
title
	^ 'Implementors of'! !

!OBListBrowser class methodsFor: 'opening' stamp: 'avi 9/17/2005 01:34'!
browseRoot: aNode
	self browseRoot: aNode title: self title! !

!OBListBrowser class methodsFor: 'opening' stamp: 'avi 9/17/2005 01:21'!
browseRoot: aNode title: aString 
	(self metaNode: self defaultMetaNode root: aNode selection: nil)
		labelPrefix: aString;
		open
	! !

!OBListBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 17:54'!
maxPanes
	^ 2! !

!OBListBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 17:54'!
minPanes
	^ 1! !

!OBListBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:58'!
defaultBackgroundColor
	^ Color lightBlue! !

!OBListBrowser methodsFor: 'accessing' stamp: 'cwp 11/25/2004 22:21'!
defaultLabel
	^ self labelPrefix, ' ', self root name printString! !

!OBListBrowser methodsFor: 'accessing' stamp: 'cwp 11/25/2004 22:18'!
labelPrefix
	^ labelPrefix ifNil: [labelPrefix _ self root metaNode edges first label capitalized, ' of']! !

!OBListBrowser methodsFor: 'accessing' stamp: 'cwp 10/17/2004 20:54'!
labelPrefix: aString
	labelPrefix _ aString! !

OBListBrowser subclass: #OBReferencesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBReferencesBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:22'!
defaultMetaNode
	| class method |
	class := OBMetaNode named: 'Class'.
	method := OBMetaNode named: 'References'.
	
	class
		childAt: #users put: method;
		addActor: OBNodeActor new.
	method
		displaySelector: #fullName;
		addActor: OBNodeActor new.
		
	^ class! !

!OBReferencesBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:34'!
title
	^ 'References to'! !

OBListBrowser subclass: #OBSendersBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBSendersBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:27'!
chaseSenders
	| send message implementor |
	send := OBMetaNode named: 'Send'.
	message := OBMetaNode named: 'Message'.
	implementor := OBMetaNode named: 'Implementor'.
	send
		displaySelector: #fullName;
		childAt: #senders put: send;
		childAt: #messages put: message;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	message
		displaySelector: #fullName;
		childAt: #implementors put: implementor;
		childAt: #sendersOfMessage labeled: 'senders' put: send;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	implementor
		childAt: #senders put: send;
		childAt: #messages put: message;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	^ send! !

!OBSendersBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:27'!
defaultMetaNode
	| selector senders |
	selector := OBMetaNode named: 'Selector'.
	senders := OBMetaNode named: 'Senders'.
	
	selector 
		childAt: #senders labeled: 'list' put: senders;
		childAt: #senders labeled: 'chase' put: self chaseSenders;
		filterClass: OBModalFilter.
	senders 
		displaySelector: #fullName;
		addActor: OBNodeActor new.

	^ selector! !

!OBSendersBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:36'!
title
	^ 'Senders of'! !

OBListBrowser subclass: #OBVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBVariablesBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:22'!
defaultMetaNode
	| class method message var |
	class := OBMetaNode named: 'Class'.
	var := OBMetaNode named: 'Variable'.
	method := OBMetaNode named: 'Method'.
	message := OBMetaNode named: 'Message'.
	class
		addActor: OBNodeActor new;
		childAt: #instanceVariables labeled: 'instance' put: var;
		childAt: #classVariables labeled: 'class' put: var;
		filterClass: OBModalFilter.
	var
		childAt: #accessors put: method;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	method
		displaySelector: #fullName;
		addActor: OBNodeActor new;
		childAt: #senders put: message;
		filterClass: OBModalFilter.
	message
		displaySelector: #fullName;
		addActor: OBNodeActor new;
		childAt: #senders put: message;
		filterClass: OBModalFilter.
	
	^ class! !

!OBVariablesBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:35'!
title
	^ 'Variables of'! !

OBCodeBrowser subclass: #OBSystemBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBSystemBrowser commentStamp: 'cwp 1/7/2005 23:56' prior: 0!
This is the basic system browser that is the work-horse of Smalltalk development tools. It presents four panes showing class categories, classes, method categories and methods.!

!OBSystemBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:13'!
alphabeticalMetaNode
	^ self
		imageClass: #classes
		comment: #comments
		metaclass: #metaclasses
! !

!OBSystemBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:54'!
defaultMetaNode
	^self hierarchicalMetaNode! !

!OBSystemBrowser class methodsFor: 'configuration' stamp: 'cwp 12/6/2004 22:02'!
defaultRootNode
	^ OBEnvironmentNode forImage! !

!OBSystemBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:54'!
hierarchicalMetaNode
	^self 
		imageClass: #classesHierarchically
		comment: #commentsHierarchically
		metaclass: #metaclassesHierarchically! !

!OBSystemBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:09'!
imageClass: classSel comment: commentSel metaclass: metaclassSel 
	| env classCategory |
	env := OBMetaNode named: 'Environment'.
	classCategory := OBMetaNode named: 'ClassCategory'.
	env childAt: #categories put: classCategory.
	self addTo: classCategory class: classSel comment: commentSel metaclass: metaclassSel.
	^env! !

!OBSystemBrowser class methodsFor: 'initializing' stamp: ''!
initialize
	self registerInOpenMenu! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:32'!
onClass: aClass
	^ self selection: (OBClassNode on: aClass)! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/15/2004 21:42'!
onClass: aClass category: aSymbol
	^ self selection: (OBMethodCategoryNode on: aSymbol inClass: aClass)! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 17:38'!
onClass: aClass selector: aSelector
	^ self selection: (OBMethodNode on: aSelector inClass: aClass)! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/15/2004 21:45'!
onEnvironment: anEnvironment category: aSymbol
	^ self selection: (OBClassCategoryNode 
						on: aSymbol 
						inEnvironment: anEnvironment)! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/5/2004 18:32'!
openOnClass: aClass
	^ (self onClass: aClass) open! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/15/2004 21:47'!
openOnClass: aClass category: aSymbol
	^ (self onClass: aClass category: aSymbol) open! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/5/2004 17:38'!
openOnClass: aClass selector: aSelector
	^ (self onClass: aClass selector: aSelector) open! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/15/2004 21:54'!
openOnEnvironment: anEnvironment category: aSymbol
	^ (self onEnvironment: anEnvironment category: aSymbol) open! !

!OBSystemBrowser class methodsFor: 'initializing' stamp: ''!
registerInOpenMenu
	TheWorldMenu registerOpenCommand: {'Image Browser'. {self. #open}}.
! !

!OBSystemBrowser class methodsFor: 'configuration' stamp: ''!
title
	^ 'System Browser'! !

!OBSystemBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:52'!
defaultBackgroundColor
	^ Color lightGreen

! !

OBCodeBrowser subclass: #OBVersionBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBVersionBrowser commentStamp: 'cwp 1/7/2005 23:58' prior: 0!
OBVersionBrowser displays a list of OBMethodVersions, which represent versions of a method present in the source or changes files. !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:09'!
defaultMetaNode
	| version |
	version := OBMetaNode named: 'Version'.
	version addActor: OBNodeActor new.
	^ (OBMetaNode named: 'Method') 
		childAt: #versions put: version.! !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'cwp 11/27/2004 22:32'!
maxPanes
	^ 1! !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'cwp 11/27/2004 22:32'!
minPanes
	^ 1! !

!OBVersionBrowser class methodsFor: 'opening' stamp: 'cwp 12/5/2004 15:42'!
openOn: aMethodVersionNode
	^ (self root: aMethodVersionNode) open! !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'cwp 11/27/2004 22:34'!
optionalButtons
	^ #('browse' 'revert')! !

!OBVersionBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:52'!
defaultBackgroundColor
	^ Color lightMagenta! !

!OBVersionBrowser methodsFor: 'accessing' stamp: 'cwp 11/27/2004 22:28'!
defaultLabel
	^ 'Versions of ', self root name printString! !

OBNode subclass: #OBCodeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBCodeNode commentStamp: 'cwp 1/8/2005 11:12' prior: 0!
OBCodeNode is an abstract superclass for node classes that represent program elements active in the image. Though it provides little functionality, it exists for structural purposes.!

OBCodeNode subclass: #OBClassAwareNode
	instanceVariableNames: 'theClass superior'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassAwareNode commentStamp: 'cwp 1/8/2005 11:23' prior: 0!
OBClassAware node models program elements that are part of a class. It provides methods for manipulating the class, as well as methods relating to sorting according to class hierarchy.

iVars

theClass 	- the class that this node is part of
superior 	- during hierarchical sorting this refers to the nearest superclass
			  that belongs to the group being sorted!

!OBClassAwareNode class methodsFor: 'sorting' stamp: 'cwp 12/12/2004 22:13'!
sortHierarchically: nodes
	nodes do: [:a | nodes do: [:b | a adoptSuperior: b]].
	nodes sort: [:a : b | a sortsBefore: b].
	^ nodes! !

!OBClassAwareNode methodsFor: 'superiors' stamp: 'cwp 12/11/2004 22:04'!
adoptSuperior: other 
	| descent |
	descent := self theClass allSuperclasses reversed.
	(descent indexOf: other theClass) > (descent indexOf: self superiorClass) 
		ifTrue: [superior := other]! !

!OBClassAwareNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:29'!
browse
	OBSystemBrowser openOnClass: self theClass! !

!OBClassAwareNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 22:43'!
browseHierarchically
	OBHierarchyBrowser openOnClass: self theClass! !

!OBClassAwareNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 23:10'!
browseHierarchyAction
	^ self 
		action: #browseHierarchically 
		buttonLabel: 'hierarchy' 
		menuLabel: 'browse hierarchy'.
! !

!OBClassAwareNode methodsFor: 'actions' stamp: 'avi 9/17/2005 01:34'!
chaseVars
	OBVariablesBrowser browseRoot: self classNode! !

!OBClassAwareNode methodsFor: 'nodes' stamp: ''!
classNode
	^ OBClassNode on: self theClass! !

!OBClassAwareNode methodsFor: 'displaying' stamp: 'cwp 1/7/2005 22:13'!
indent
	| size indent |
	size := 0.
	self superiorsDo: [:ea | size := size + 1].
	indent := String new: size * 2.
	indent atAllPut: $ .
	^ indent! !

!OBClassAwareNode methodsFor: 'displaying' stamp: 'cwp 1/7/2005 22:21'!
indentedName
	^ self indent, self name! !

!OBClassAwareNode methodsFor: 'ancestry' stamp: 'cwp 12/5/2004 12:56'!
isDescendantOfClassCat: aClassCategoryNode

	"optimized version: sending #category to the class is slow"
	^ (self theNonMetaClass environment organization 
		listAtCategoryNamed: aClassCategoryNode name)
			includes: self theNonMetaClassName

! !

!OBClassAwareNode methodsFor: 'ancestry' stamp: ''!
isDescendantOfClass: aClassNode
	^ self theClassName = aClassNode theClassName! !

!OBClassAwareNode methodsFor: 'superiors' stamp: 'cwp 12/11/2004 22:04'!
sortsBefore: aClassNode
	| own other |
	own := self withSuperiors.
	other := aClassNode withSuperiors.
	1 	to: (own size min: other size)
		do: [:i | (own at: i) == (other at: i) ifFalse: 
				[^ (own at: i) theClassName <= (other at: i) theClassName]].
	^ other includes: self
	! !

!OBClassAwareNode methodsFor: 'superiors' stamp: 'cwp 12/11/2004 22:03'!
superior
	^ superior! !

!OBClassAwareNode methodsFor: 'superiors' stamp: 'cwp 12/11/2004 22:03'!
superiorClass
	^superior ifNotNil: [superior theClass]! !

!OBClassAwareNode methodsFor: 'superiors' stamp: 'cwp 12/11/2004 22:03'!
superiors
	| result |
	result := OrderedCollection new.
	self superiorsDo: [:ea | result add: ea].
	^ result asArray reversed! !

!OBClassAwareNode methodsFor: 'superiors' stamp: 'cwp 12/11/2004 22:03'!
superiorsDo: aBlock
	| cursor |
	cursor := superior.
	[cursor isNil] whileFalse:
		[aBlock value: cursor.
		cursor := cursor superior]! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theClass
	^ theClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theClassName
	^ self theClass name! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theClass: aClass
	theClass := aClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theMetaClass
	^ self theClass theMetaClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theNonMetaClass
	^ self theClass theNonMetaClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theNonMetaClassName
	^ self theNonMetaClass name! !

!OBClassAwareNode methodsFor: 'superiors' stamp: 'cwp 12/11/2004 22:03'!
withSuperiors
	| result |
	result := OrderedCollection new.
	result add: self.
	self superiorsDo: [:ea | result add: ea].
	^ result asArray reversed! !

OBClassAwareNode subclass: #OBClassCommentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassCommentNode commentStamp: 'cwp 1/8/2005 11:24' prior: 0!
OBClassCommentNode represents the comment attached to a particular class. !

!OBClassCommentNode class methodsFor: 'as yet unclassified' stamp: ''!
on: classRef
	^ self new setClass: classRef! !

!OBClassCommentNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 23:03'!
actions
	^ {
		self browseAction.
		self browseHierarchyAction.
		self action: #chaseVars buttonLabel: 'variables' menuLabel: 'chase variables'
	}! !

!OBClassCommentNode methodsFor: 'public' stamp: 'cwp 12/13/2004 00:50'!
indentedName
	^ self indent, self name! !

!OBClassCommentNode methodsFor: 'public' stamp: 'cwp 12/13/2004 00:51'!
name
	^ self theClass name! !

!OBClassCommentNode methodsFor: 'initializing' stamp: ''!
setClass: aClass
	self theClass: aClass! !

!OBClassCommentNode methodsFor: 'definition' stamp: ''!
text
	^ self theClass comment! !

!OBClassCommentNode methodsFor: 'definition' stamp: ''!
text: aText
	self theClass comment: aText stamp: Utilities changeStamp.
	^ true! !

OBClassAwareNode subclass: #OBClassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassNode commentStamp: 'cwp 1/8/2005 11:27' prior: 0!
OBClassNode wraps a class for display in various types of code browsers. It provides many navigation methods for the different types of browsers where classes may appear.!

!OBClassNode class methodsFor: 'instance creation' stamp: 'cwp 11/30/2004 21:06'!
on: aClass 
	^ self new setClass: aClass ! !

!OBClassNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 23:01'!
actions
	^ {
		self action: #findMethod withLabel: 'find method...' withKeystroke: $f withIcon: MenuIcons findIcon.
		self action: #fileOut withLabel: 'file out'.
		self browseAction.
		self browseHierarchyAction.
		self action: #chaseVars buttonLabel: 'variables' menuLabel: 'chase variables'.
		self action: #inspectInstances withLabel: 'inspect instances'.
		self action: #inspectSubInstances withLabel: 'inspect subinstances'
	}! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 1/7/2005 23:05'!
allCategory
	^ Array with: (OBAllMethodCategoryNode on: self theClass)! !

!OBClassNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfClass:! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 1/7/2005 23:06'!
categories
	^ self theClass organization categories
		collect: [:cat | OBMethodCategoryNode on: cat inClass: self theClass]
			! !

!OBClassNode methodsFor: 'nodes' stamp: ''!
classCategoryNode
	^ OBClassCategoryNode 
		on: self theNonMetaClass category
		inEnvironment: self theClass environment! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 23:31'!
classHierarchy
	^ self nodeHierarchyWithClass: OBClassNode! !

!OBClassNode methodsFor: 'navigating' stamp: ''!
classVariables
	^ self theClass allClassVarNames asArray sort
		collect: [:ea | OBClassVariableNode on: ea inClass: self theClass]! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 23:31'!
commentHierarchy
	^ self nodeHierarchyWithClass: OBClassCommentNode! !

!OBClassNode methodsFor: 'accessing' stamp: ''!
definition
	^ OBClassDefinition 
		environment: self theClass environment 
		template: self theClass definition! !

!OBClassNode methodsFor: 'drag and drop' stamp: ''!
dropOnClassCategory: aNode
	self theClass category: aNode name.
	self signalSelection! !

!OBClassNode methodsFor: 'actions' stamp: 'cwp 10/26/2004 00:49'!
fileOut
	self theNonMetaClass fileOut! !

!OBClassNode methodsFor: 'actions' stamp: ''!
findMethod
	| selectors selection |
	selectors := self theClass selectors asSortedArray.
	selectors isEmpty ifTrue: [^nil].
	selection := OBChoiceRequest labels: selectors.
	selection ifNotNil: [(OBMethodNode 
									on: selection
									inClass: self theClass) signalSelection].! !

!OBClassNode methodsFor: 'actions' stamp: ''!
inspectInstances
	self theNonMetaClass inspectAllInstances! !

!OBClassNode methodsFor: 'actions' stamp: ''!
inspectSubInstances
	self theNonMetaClass inspectSubInstances! !

!OBClassNode methodsFor: 'navigating' stamp: ''!
instanceVariables
	^ self theClass allInstVarNames asArray sort
		collect: [:ea | OBInstanceVariableNode on: ea inClass: self theClass]! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 23:30'!
metaclassHierarchy
	^ self nodeHierarchyWithClass: OBMetaclassNode! !

!OBClassNode methodsFor: 'displaying' stamp: 'cwp 12/13/2004 00:51'!
name
	^ self theClass name! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 23:30'!
nodeHierarchyWithClass: aClass
	^ OBClassAwareNode sortHierarchically: 
		(self surroundingHierarchy collect: [:ea | aClass on: ea])! !

!OBClassNode methodsFor: 'accessing' stamp: ''!
organization
	^ self theClass organization! !

!OBClassNode methodsFor: 'printing' stamp: 'cwp 12/10/2004 23:53'!
printOn: aStream
	aStream
		print: self class;
		nextPut: $<;
		print: self theClass;
		nextPut: $>! !

!OBClassNode methodsFor: 'initializing' stamp: ''!
setClass: aClass
	self theClass: aClass! !

!OBClassNode methodsFor: 'accessing' stamp: 'cwp 12/12/2004 23:27'!
surroundingHierarchy
	 ^ (self theClass withAllSuperclasses union: self theClass allSubclasses)
		asArray! !

!OBClassNode methodsFor: 'displaying' stamp: ''!
title
	^ self name! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:45'!
users
	^ (SystemNavigation default allCallsOn: (theClass environment associationAt: theClass name))
		collect: [:ref | OBClassRefNode on: self name inMethod: ref]! !

!OBClassNode methodsFor: 'accessing' stamp: 'cwp 12/12/2004 22:45'!
= other
	self species = other species ifFalse: [^ false].
	^ self theClass = other theClass! !

OBClassNode subclass: #OBMetaclassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMetaclassNode commentStamp: 'cwp 1/8/2005 11:29' prior: 0!
OBMetaclassNode is essentially the same as an OBClassNode, but overrides a few methods to work properly with metaclasses.!

!OBMetaclassNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:57'!
browse
	^ OBSystemBrowser openOnClass: self theNonMetaClass ! !

!OBMetaclassNode methodsFor: 'displaying' stamp: 'cwp 12/13/2004 00:56'!
name 
	^ self nonMetaName! !

!OBMetaclassNode methodsFor: 'displaying' stamp: ''!
nonMetaName
	^ self theNonMetaClass name! !

!OBMetaclassNode methodsFor: 'initializing' stamp: ''!
setClass: aClass
	super setClass: aClass theMetaClass
		! !

OBClassAwareNode subclass: #OBMethodCategoryNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMethodCategoryNode commentStamp: 'cwp 1/8/2005 12:29' prior: 0!
OBMethodCategory represents a category within a ClassOrganization. Instead of an organization definition, OBMethodCategory presents a MethodDefinition with the class' default method template.!

OBMethodCategoryNode subclass: #OBAllMethodCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBAllMethodCategoryNode commentStamp: 'cwp 1/8/2005 12:31' prior: 0!
OBAllMethodCategory implements the synthetic '-- all --' category, which contains all the methods in a class.!

!OBAllMethodCategoryNode class methodsFor: 'as yet unclassified' stamp: ''!
on: className
	^ self on: '-- all --' inClass: className! !

!OBAllMethodCategoryNode methodsFor: 'accessing' stamp: ''!
category
	^ 'as yet unclassified'! !

!OBAllMethodCategoryNode methodsFor: 'accessing' stamp: ''!
methodReferences
	^ self theClass selectors asSortedArray 
		collect: [:ea | MethodReference new setStandardClass: self theClass methodSymbol: ea]! !

!OBMethodCategoryNode class methodsFor: 'actions' stamp: 'cwp 9/18/2004 22:58'!
actionsForParent: aNode
	^ #()! !

!OBMethodCategoryNode class methodsFor: 'instance creation' stamp: ''!
on: aString inClass: aClassReference
	^ self new setName: aString class: aClassReference! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 23:04'!
actions
	^ {
		self action: #fileOut withLabel: 'file out'.
		self browseAction.
		self browseHierarchyAction.
		self action: #chaseVars buttonLabel: 'variables' menuLabel: 'chase variables'
	}
! !

!OBMethodCategoryNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfMethodCat:! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:50'!
browse
	OBSystemBrowser openOnClass: self theClass category: name! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: ''!
category
	^ name! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: ''!
container
	^ self theClass! !

!OBMethodCategoryNode methodsFor: 'public' stamp: ''!
definition
	^ OBMethodDefinition inCategory: self category inClass: self theClass! !

!OBMethodCategoryNode methodsFor: 'drag and drop' stamp: ''!
dropSelector
	^ #dropOnMethodCategory:! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: ''!
fileOut
	self theClass fileOutCategory: name! !

!OBMethodCategoryNode methodsFor: 'comparing' stamp: 'cwp 12/20/2004 00:53'!
hash
	^ self name hash! !

!OBMethodCategoryNode methodsFor: 'ancestry' stamp: ''!
isDescendantOfMethodCat: other
	^ (other theClassName = self theClassName) and: [other name = name]! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: ''!
methodReferences
	^ (self theClass organization listAtCategoryNamed: name)
		collect: [:ea | MethodReference new
						setClassSymbol: self theNonMetaClassName
						classIsMeta: self theClass isMeta
						methodSymbol: ea
						stringVersion: '']! !

!OBMethodCategoryNode methodsFor: 'navigating' stamp: ''!
methods
	^ self methodReferences collect: [:ref | OBMethodNode on: ref]! !

!OBMethodCategoryNode methodsFor: 'public' stamp: ''!
name
	^ name! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 9/19/2004 00:11'!
remove
	self theClass removeCategory: name.
	self signalDeletion! !

!OBMethodCategoryNode methodsFor: 'initializing' stamp: ''!
setName: aString class: aClass
	name := aString.
	self theClass: aClass! !

!OBMethodCategoryNode methodsFor: 'comparing' stamp: 'cwp 12/20/2004 00:50'!
= other
	self species = other species ifFalse: [^ false].	
	self theClass = other theClass ifFalse: [^ false].
	^ self name = other name! !

OBClassAwareNode subclass: #OBMethodNode
	instanceVariableNames: 'selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMethodNode commentStamp: 'cwp 1/8/2005 12:37' prior: 0!
OBMethodNode wraps a method in a particular class. It supplies an OBMethodDefinition for displaying and editing the source code of the method, and various actions for manipulating the method. It has two roles: first it appears in the right most pane of a standard system browser. Second, it often serves as the root node for an OBListBrowser, and has navigation methods for senders, implementors etc.

OBMethodNode also serves as the superclass for nodes that represent *parts* of a method, such as sends of a particular message, references to a class, accesses to a instance variable, etc.!

OBMethodNode subclass: #OBClassRefNode
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassRefNode commentStamp: 'cwp 1/8/2005 12:38' prior: 0!
Each instance of OBClassRefNode refers to a reference to a particular class from the source code of a method. It's used in the OBListBrowser created by the 'browse references' class action.!

!OBClassRefNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/1/2004 00:50'!
on: aClassName inMethod: aMethodRef
	^ self new setClassName: aClassName reference: aMethodRef! !

!OBClassRefNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 22:14'!
name
	^ className! !

!OBClassRefNode methodsFor: 'accessing' stamp: 'cwp 12/1/2004 01:16'!
selection
	| start parser |
	(parser _ Compiler parserClass new) parseSelector: self source.
	start := parser endOfLastToken.
	start := (self source asString findString: className startingAt: start).
	^ start to: start + className size - 1! !

!OBClassRefNode methodsFor: 'initialize-release' stamp: 'cwp 12/1/2004 01:10'!
setClassName: theName reference: aMethodRef
	self setReference: aMethodRef.
	className := theName.
! !

OBMethodNode subclass: #OBMessageNode
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMessageNode commentStamp: 'cwp 1/8/2005 12:40' prior: 0!
An OBMessageNode refers to a message send in the source code of a method. They are typically used in the 'senders' browser.!

!OBMessageNode class methodsFor: 'instance creation' stamp: ''!
fromMethodNode: aMethodNode
	^ self new 
		setMessage: aMethodNode selector
		selector: aMethodNode selector
		class: aMethodNode theClass! !

!OBMessageNode class methodsFor: 'instance creation' stamp: ''!
on: aSelector inMethodNode: aNode
	^ self 
		on: aSelector 
		inMethod: aNode selector 
		inClass: aNode theClass! !

!OBMessageNode class methodsFor: 'instance creation' stamp: 'cwp 12/8/2004 23:46'!
on: aSelector inMethodReference: ref
	^ self 
		on: aSelector 
		inMethod: ref methodSymbol
		inClass: ref actualClass! !

!OBMessageNode class methodsFor: 'instance creation' stamp: ''!
on: aSelector inMethod: aSelector2 inClass: classRef
	^ self new setMessage: aSelector selector: aSelector2 class: classRef! !

!OBMessageNode methodsFor: 'navigating' stamp: ''!
implementors
	^ self implementorsOf: message! !

!OBMessageNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:26'!
name
	^ message! !

!OBMessageNode methodsFor: 'accessing' stamp: ''!
selection
	| methodNode assoc |
	methodNode := Parser new 
					parse: self source
					class: self theClass.
	assoc := (methodNode encoder rawSourceRanges) 
			associations detect: [:ea | ea key isMessage: message receiver: nil arguments: nil]
						ifNone: [nil -> (1 to: 0)].
						"Some messages are generated by the compiler."
	^ assoc value! !

!OBMessageNode methodsFor: 'navigating' stamp: ''!
sendersOfMessage
	^ self sendersOf: message! !

!OBMessageNode methodsFor: 'initializing' stamp: ''!
setMessage: aSelector selector: aSelector2 class: aClass
	message := aSelector.
	selector := aSelector2.
	self theClass: aClass! !

!OBMethodNode class methodsFor: 'instance creation' stamp: ''!
on: aMethodReference
	^ self new setReference: aMethodReference! !

!OBMethodNode class methodsFor: 'instance creation' stamp: ''!
on: aSelector inClass: aClassReference
	^ self new setSelector: aSelector class: aClassReference! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 23:05'!
actions
	^{ 
		self action: #fileOut withLabel: 'file out'.
		self browseAction.
		self browseHierarchyAction.
		self 
			action: #browseSenders
			withMenuLabel: 'senders'
			withButtonLabel: 'senders'
			withKeystroke: $n
			withIcon: MenuIcons findIcon.
		self 
			action: #browseImplementors
			withMenuLabel: 'implementors'
			withButtonLabel: 'implementors'
			withKeystroke: $m
			withIcon: MenuIcons findIcon.
		self 
			action: #browseVersions
			buttonLabel: 'versions'
			menuLabel: 'versions'.
		self 
			action: #browseInheritance
			buttonLabel: 'inheritance'
			menuLabel: 'inheritance'.
		self 
			action: #chaseVars
			buttonLabel: 'variables'
			menuLabel: 'chase variables'.
		self 
			action: #remove
			withLabel: 'remove'
			withKeystroke: $x
			withIcon: MenuIcons deleteIcon}! !

!OBMethodNode methodsFor: 'private' stamp: ''!
addOverridersOf: aSelector inClass: aClass to: aCollection
	aClass subclasses do: 
		[:ea | (ea includesSelector: aSelector)
					ifTrue: [aCollection add: ea]
					ifFalse: [self addOverridersOf: aSelector inClass: ea to: aCollection]]! !

!OBMethodNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfMethod: ! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 12/5/2004 18:35'!
browse
	OBSystemBrowser openOnClass: self theClass selector: selector! !

!OBMethodNode methodsFor: 'actions' stamp: 'avi 9/17/2005 01:29'!
browseClass: aClass
	aClass browseRoot: self copy! !

!OBMethodNode methodsFor: 'actions' stamp: 'avi 9/17/2005 01:28'!
browseImplementors
	self browseClass: OBImplementorsBrowser! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 12/5/2004 18:03'!
browseInheritance
	OBInheritanceBrowser openRoot: self inheritanceRoot selection: self! !

!OBMethodNode methodsFor: 'actions' stamp: 'avi 9/17/2005 01:28'!
browseSenders
	self browseClass: OBSendersBrowser! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 12/5/2004 15:42'!
browseVersions
	OBVersionBrowser openOn: self copy! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 12/1/2004 00:36'!
definition
	^ OBMethodDefinition
		selection: self selection
		source: self source
		inClass: self theClass! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 10/18/2004 00:29'!
doRemove
	self theClass removeSelector: self selector.
	self signalDeletion! !

!OBMethodNode methodsFor: 'drag and drop' stamp: ''!
dropOnMethodCategory: aNode

	"We don't support dropping on another class yet."
	aNode theClass = self theClass ifFalse: [^ false].
	
	self theClass organization classify: self selector under: aNode name.
	OBListChanged signal.
	
	^ true! !

!OBMethodNode methodsFor: 'actions' stamp: ''!
fileOut
	self theClass fileOutMethod: selector! !

!OBMethodNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:08'!
fullName
	^ self theClassName, '>>', selector! !

!OBMethodNode methodsFor: 'comparing' stamp: 'cwp 12/8/2004 22:07'!
hash
	^ theClass hash bitXor: selector hash! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 1/2/2005 23:12'!
implementors
	^ self implementorsOf: selector! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 1/7/2005 22:35'!
implementorsHierarchically
	^ OBClassAwareNode sortHierarchically: self implementors! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 12/8/2004 23:48'!
implementorsOf: aSelector
	^ (SystemNavigation default allImplementorsOf: aSelector) asSortedArray
			collect: [:ref | OBMethodNode on: ref]! !

!OBMethodNode methodsFor: 'displaying' stamp: 'cwp 12/20/2004 23:13'!
indentedFullName
	^ self indent, self fullName! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 12/8/2004 23:49'!
inheritanceRoot
	| rootClass |
	rootClass := (self theClass withAllSuperclasses asArray
					select: [:ea | ea includesSelector: self selector]) last.
	^ OBCollectionNode on: {OBMethodNode 
								on: self selector 
								inClass: rootClass}! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 12/2/2004 23:28'!
isDescendantOfMethodCat: aMethodCatNode
	^ (self theClass organization categoryOfElement: self selector) = aMethodCatNode name! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 10/17/2004 23:55'!
isDescendantOfMethodVersion: anOBMethodVersionNode 
	^ false! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 11/28/2004 10:37'!
isDescendantOfMethod: other
	^ other selector = selector
		and: [self theClass withAllSuperclasses includes: other theClass].
	! !

!OBMethodNode methodsFor: 'nodes' stamp: ''!
messageNode
	^ OBMessageNode fromMethodNode: self! !

!OBMethodNode methodsFor: 'private' stamp: ''!
messageSelectors
	^ ((self theClass compiledMethodAt: self selector ifAbsent: [^ #()]) messages) asSortedArray! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
messages
	^ (((self theClass
				compiledMethodAt: self selector 
				ifAbsent: [^ #()]) messages) asSortedArray)
					collect: [:sel | OBMessageNode 
									on: sel 
									inMethod: selector 
									inClass: self theClass]! !

!OBMethodNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:08'!
name
	^ selector! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:49'!
overrides
	| classes |
	classes := OrderedCollection new.
	self addOverridersOf: self selector inClass: self theClass to: classes.
	^ classes collect: [:ea | OBMethodNode on: selector inClass: ea] ! !

!OBMethodNode methodsFor: 'printing' stamp: 'cwp 12/8/2004 21:59'!
printOn: aStream
	aStream
		nextPutAll: self class name;
		nextPut: $<;
		nextPutAll: self theClass name;
		nextPut: $#;
		nextPutAll: self selector;
		nextPut: $>! !

!OBMethodNode methodsFor: 'accessing' stamp: ''!
reference
	^ self referenceForMethod: selector ofClass: self theClassName! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 10/18/2004 00:30'!
remove
	| senders choice |
	senders _ SystemNavigation default allCallsOn: self selector.
	senders isEmpty ifTrue: [^ self doRemove].
	choice _ OBChoiceRequest
				prompt: 'This message has ', senders size asString, ' senders.'
				labels: #('Remove it' 
						'Remove, then browse senders' 
						'Don''t remove, but show me those senders' 
						'Forget it -- do nothing -- sorry I asked')
				values: #(doRemove removeAndBrowse simpleBrowseSenders nil).
	choice ifNotNil: [^ self perform: choice]
! !

!OBMethodNode methodsFor: 'private' stamp: ''!
removeAndBrowse
	self simpleBrowseSenders.
	self theClass removeSelector: self selector.
	self signalDeletion! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 12/1/2004 00:37'!
selection
	^ 1 to: 0! !

!OBMethodNode methodsFor: 'actions' stamp: ''!
selectMessage
	| messages sel |
	messages := self messageSelectors copyWithFirst: selector.
	sel := (OBChoiceRequest labels: messages).
	^ sel ifNotNil: [OBMessageNode on: sel inMethodNode: self]! !

!OBMethodNode methodsFor: 'accessing' stamp: ''!
selector
	^ selector! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
selectorAndMessages
	^ self messages copyWithFirst: self messageNode! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
senders
	^ self sendersOf: selector! !

!OBMethodNode methodsFor: 'private' stamp: ''!
sendersOf: aSelector
	^ (SystemNavigation default allCallsOn: aSelector) asSortedArray
			collect: [:ref | OBMessageNode on: aSelector inMethodReference: ref]! !

!OBMethodNode methodsFor: 'initializing' stamp: ''!
setReference: aMethodReference
	self 
		setSelector: aMethodReference methodSymbol
		class: (aMethodReference actualClass)! !

!OBMethodNode methodsFor: 'initializing' stamp: ''!
setSelector: aSelector class: aClass
	selector := aSelector.
	self theClass: aClass
! !

!OBMethodNode methodsFor: 'private' stamp: 'avi 9/17/2005 01:29'!
simpleBrowseSenders
	OBSendersBrowser browseRoot: self! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 12/1/2004 00:36'!
source
	^ (self theClass sourceCodeAt: self selector ifAbsent: [^ '']) 
		asText makeSelectorBold! !

!OBMethodNode methodsFor: 'accessing' stamp: ''!
sourceFiles
	^ OBSourceFilesRequest signal! !

!OBMethodNode methodsFor: 'private' stamp: ''!
sourcePointer
	^ (self theClass compiledMethodAt: self selector) sourcePointer! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
versions
	^ (OBMethodVersion scan: self sourceFiles from: self sourcePointer)
		collect: [:ea | OBMethodVersionNode on: ea]! !

!OBMethodNode methodsFor: 'comparing' stamp: 'cwp 12/8/2004 22:07'!
= other
	^ (self species = other species)
		and: [self theClass = other theClass] 
		and: [self selector = other selector]! !

OBClassAwareNode subclass: #OBMethodVersionNode
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMethodVersionNode commentStamp: 'cwp 1/8/2005 12:41' prior: 0!
OBMethodVersions refer to OBMethodVersions, and are used by the VersionBrowser.!

!OBMethodVersionNode class methodsFor: 'instance creation' stamp: ''!
on: aMethodVersion 
	^ self new setVersion: aMethodVersion! !

!OBMethodVersionNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 22:38'!
actions
	^ {
		self browseAction.
	  	self action: #revert buttonLabel: 'revert' menuLabel: 'revert to selected version'
	}! !

!OBMethodVersionNode methodsFor: 'ancestry' stamp: 'cwp 10/17/2004 23:41'!
ancestrySelector
	^ #isDescendantOfMethodVersion:! !

!OBMethodVersionNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:51'!
browse
	OBSystemBrowser openOnClass: self theClass selector: version selector! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:57'!
category
	^ self theClass organization categoryOfElement: version selector! !

!OBMethodVersionNode methodsFor: 'compiling' stamp: 'cwp 10/17/2004 23:53'!
definition
	^ (OBMethodDefinition source: version source inClass: self theClass)
		callback: [:sel | 
					version selector = sel
						ifTrue: [self class on: version latest]
						ifFalse: [OBMethodNode on: sel inClass: self theClass]]! !

!OBMethodVersionNode methodsFor: 'ancestry' stamp: 'cwp 10/17/2004 23:43'!
isDescendantOfMethodVersion: aNode
	^ (version = aNode version)! !

!OBMethodVersionNode methodsFor: 'displaying' stamp: 'cwp 11/27/2004 21:38'!
name
	| stamp |
	stamp := version stamp ifNil: ['<timestamp missing>'].
	^ version theClassName, '>>', version selector, ' ', stamp! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: ''!
reference
	^ self referenceForMethod: version selector ofClass: self theClassName.! !

!OBMethodVersionNode methodsFor: 'actions' stamp: ''!
revert
	version fileIn! !

!OBMethodVersionNode methodsFor: 'initializing' stamp: ''!
setVersion: aMethodVersion
	version := aMethodVersion.
	self theClass: version theClass! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:44'!
version
	^ version! !

OBClassAwareNode subclass: #OBVariableNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBVariableNode commentStamp: 'cwp 1/8/2005 12:46' prior: 0!
OBVariableNode is an abstract superclass for the two types of variables a class can contain - class variables and instance variables. The only difference between the two subclasses is how they search for methods that refer to them.!

OBVariableNode subclass: #OBClassVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassVariableNode commentStamp: 'cwp 1/8/2005 12:51' prior: 0!
OBClassVariable provides a method for finding methods which refer to a shared variable, ie, by searching for an association in the literal frame rather than for bytecodes refering to an instance variable.!

!OBClassVariableNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:47'!
accessors
	| literal |
	literal := (self theClass withAllSuperclasses 
				gather: [:ea | ea classPool associations])
					detect: [:ea | ea key = name].
	^ ((self systemNavigation allCallsOn: literal) asArray sort)
		collect: [:ref | OBMethodNode on: ref]! !

OBVariableNode subclass: #OBInstanceVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBInstanceVariableNode commentStamp: 'cwp 1/8/2005 12:50' prior: 0!
OBClassVariable provides a method for finding methods which refer to an instance, ie, by searching for instance variable access bytecodes rather than for an association in the literal frame.!

!OBInstanceVariableNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:48'!
accessors
	| accessors |
	accessors := OrderedCollection new.
	self theClass withAllSubAndSuperclassesDo: [:class |
		(class whichSelectorsAccess: name) asSortedCollection
			do: [:ea | ea = #DoIt ifFalse: [accessors add: 
					(self referenceForMethod: ea ofClass: class name)]]].
	^ accessors asArray	collect: [:ref | OBMethodNode on: ref]! !

!OBVariableNode class methodsFor: 'instance creation' stamp: ''!
on: instVarName inClass: aClassReference 
	^ self new setName: instVarName class: aClassReference! !

!OBVariableNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 22:38'!
actions
	^ {self browseAction}! !

!OBVariableNode methodsFor: 'displaying' stamp: ''!
name
	^ name! !

!OBVariableNode methodsFor: 'initializing' stamp: ''!
setName: aString class: aClass
	name := aString.
	self theClass: aClass! !

OBCodeNode subclass: #OBClassCategoryNode
	instanceVariableNames: 'environment name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassCategoryNode commentStamp: 'cwp 1/8/2005 12:58' prior: 0!
OBClassCategory represents a system category in the image's SystemOrganization.!

!OBClassCategoryNode class methodsFor: 'actions' stamp: ''!
actionsForParent: aNode
	^ Array with: (OBAction
						label: 'find class...'
						receiver: self
						selector: #findClassIn:
						arguments: (Array with: aNode environment)
						keystroke: $f)
! !

!OBClassCategoryNode class methodsFor: 'actions' stamp: 'cwp 1/8/2005 14:16'!
findClassIn: anEnvironment
	| pattern class |
	pattern := OBTextRequest 
					prompt: 'Please type the name or fragment to look for' 
					template: ''.
	pattern ifNil: [^self].
	class := self findClassIn: anEnvironment pattern: pattern.
	class ifNotNil: [(OBClassNode on: class) signalSelection].! !

!OBClassCategoryNode class methodsFor: 'actions' stamp: ''!
findClassIn: anEnvironment pattern: pattern
	| classNames className toMatch potentialClassNames |
	toMatch := (pattern copyWithout: $.) asLowercase.
	potentialClassNames := anEnvironment classNames asArray.
	classNames := pattern last = $.
					ifTrue: [potentialClassNames
							select: [:nm | nm asLowercase = toMatch]]
					ifFalse: [potentialClassNames
							select: [:n | n includesSubstring: toMatch caseSensitive: false]].
	classNames size = 0 ifTrue: [^ nil].
	classNames size = 1 ifTrue: [^ anEnvironment at: classNames first asSymbol].
	className := self userSelectionOf: classNames for: (pattern copyWithout: $.) asLowercase.
	^ className ifNotNil: [anEnvironment at: className asSymbol]! !

!OBClassCategoryNode class methodsFor: 'instance creation' stamp: ''!
on: aString
	^ self new setName: aString! !

!OBClassCategoryNode class methodsFor: 'instance creation' stamp: ''!
on: aString inEnvironment: anEnvironment
	^ self new setName: aString environment: anEnvironment! !

!OBClassCategoryNode class methodsFor: 'actions' stamp: ''!
userSelectionOf: classNames for: toMatch 
	| exactMatch lines index |
	exactMatch := classNames	detect: [:ea | ea asLowercase = toMatch] 
								ifNone: [nil].
	lines := #().
	exactMatch ifNotNil:	[classNames copyWithFirst: exactMatch.
							lines := #(1)].
	index := (PopUpMenu labelArray: classNames lines: lines) startUp.
	index isZero
		ifTrue: [^ nil].
	^ (classNames at: index) asSymbol! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 22:40'!
actions
	^ {
		self action: #fileOut withLabel: 'file out'.
		self browseAction.
	}
! !

!OBClassCategoryNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfClassCat:! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:44'!
browse
	OBSystemBrowser openOnEnvironment: environment category: name! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: ''!
classes
	^ self classNames collect: [:ea | OBClassNode on: (environment at: ea)]! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 22:12'!
classesHierarchically
	^ OBClassAwareNode sortHierarchically: self classes! !

!OBClassCategoryNode methodsFor: 'private' stamp: ''!
classNames
	^ environment organization listAtCategoryNamed: name! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: ''!
comments
	^ self classNames collect: [:ea | OBClassCommentNode on: (environment at: ea)]! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 22:19'!
commentsHierarchically
	^ OBClassAwareNode sortHierarchically: self comments! !

!OBClassCategoryNode methodsFor: 'accessing' stamp: 'cwp 9/22/2004 22:13'!
container
	^ environment! !

!OBClassCategoryNode methodsFor: 'displaying' stamp: ''!
definition
	^ OBClassDefinition 
		environment: environment 
		template: ((environment at: #Class) template: name)! !

!OBClassCategoryNode methodsFor: 'drag and drop' stamp: ''!
dropSelector
	^ #dropOnClassCategory:! !

!OBClassCategoryNode methodsFor: 'actions' stamp: ''!
fileOut
	environment organization fileOutCategory: name.! !

!OBClassCategoryNode methodsFor: 'ancestry' stamp: ''!
isDescendantOfClassCat: other
	^ other name = name! !

!OBClassCategoryNode methodsFor: 'class creation' stamp: ''!
isRedefinedBy: definition
	"No class is selected, so the definition can't be a redefinition."
	
	^ false! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: ''!
metaclasses
	^ self classNames collect: [:ea | OBMetaclassNode on: (environment at: ea)]! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 22:19'!
metaclassesHierarchically
	^ OBClassAwareNode sortHierarchically: self metaclasses! !

!OBClassCategoryNode methodsFor: 'displaying' stamp: ''!
name
	^ name! !

!OBClassCategoryNode methodsFor: 'actions' stamp: ''!
printOut
	environment organization fileOutCategory: name asHtml: true! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 9/19/2004 00:11'!
remove
	environment organization removeSystemCategory: name.
	self signalDeletion! !

!OBClassCategoryNode methodsFor: 'initializing' stamp: ''!
setName: aString
	self setName: aString environment: self class environment! !

!OBClassCategoryNode methodsFor: 'initializing' stamp: ''!
setName: aString environment: anEnvironment
	name := aString asSymbol.
	environment := anEnvironment! !

!OBClassCategoryNode methodsFor: 'displaying' stamp: ''!
text
	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self name, ''''! !

!OBCodeNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 22:31'!
browseAction
	^ self 
		action: #browse
		withMenuLabel: 'browse'
		withButtonLabel: 'browse'
		withKeystroke: $b
		withIcon: MenuIcons windowIcon.
! !

OBCodeNode subclass: #OBEnvironmentNode
	instanceVariableNames: 'environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBEnvironmentNode commentStamp: 'cwp 1/8/2005 13:01' prior: 0!
OBEnvironmentNode wraps an instance of SystemDictionary. In current Squeak images, there is only one such instance, but OB-Standard is coded so as to use rely on this assumption as little as possible. Thus OBEnvironmentNode typically serves as the root of the standard browser, and passes its environment on to other nodes in the tree.!

!OBEnvironmentNode class methodsFor: 'as yet unclassified' stamp: ''!
forImage
	^ self on: Smalltalk! !

!OBEnvironmentNode class methodsFor: 'as yet unclassified' stamp: ''!
on: anEnvironment
	^ self new setEnvironment: anEnvironment! !

!OBEnvironmentNode methodsFor: 'navigating' stamp: ''!
categories
	^ environment organization categories collect: [:cat | OBClassCategoryNode 
															on: cat
															inEnvironment: environment]! !

!OBEnvironmentNode methodsFor: 'accessing' stamp: ''!
environment
	^ environment! !

!OBEnvironmentNode methodsFor: 'displaying' stamp: ''!
name 
	^ 'Squeak'! !

!OBEnvironmentNode methodsFor: 'accessing' stamp: ''!
organization
	^ self environment organization! !

!OBEnvironmentNode methodsFor: 'initializing' stamp: ''!
setEnvironment: anEnvironment
	environment := anEnvironment! !

OBCodeNode subclass: #OBSelectorNode
	instanceVariableNames: 'selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBSelectorNode commentStamp: 'cwp 1/8/2005 13:03' prior: 0!
OBSelectorNode wraps an instance of Symbol. It typically serves as the root of a senders or implementors OBListBrowser, and is used when cmd-n or cmd-m is invoke from the definition panel, and so no OBMethodNode is available.!

!OBSelectorNode class methodsFor: 'instance creation' stamp: 'cwp 10/31/2004 01:11'!
on: aSelector
	^ self new selector: aSelector! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:48'!
implementors
	^ (SystemNavigation default allImplementorsOf: self selector) asSortedArray
			collect: [:ref | OBMethodNode on: ref]! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 1/7/2005 22:35'!
implementorsHierarchically
	^ OBClassAwareNode sortHierarchically: self implementors! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 10/31/2004 01:12'!
name
	^ selector printString! !

!OBSelectorNode methodsFor: 'accessing' stamp: 'cwp 10/31/2004 01:11'!
selector
	^ selector! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 10/31/2004 01:12'!
selectorAndMessages
	^ Array with: self! !

!OBSelectorNode methodsFor: 'accessing' stamp: 'cwp 10/31/2004 01:10'!
selector: aSelector
	selector := aSelector! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 11/1/2004 02:22'!
senders
	^ (SystemNavigation default allCallsOn: self selector) asSortedArray
			collect: [:ref | OBMessageNode on: self selector inMethodReference: ref]! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 10/31/2004 01:48'!
sendersOfMessage
	^ (SystemNavigation default allCallsOn: self selector) asSortedArray
			collect: [:ref | OBMessageNode on: self selector inMethodReference: ref]! !

!Metaclass methodsFor: '*ob-standard-converting' stamp: ''!
asNode
	^ (OBMetaclassNode on: self)! !

Notification subclass: #OBSourceFilesRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBSourceFilesRequest commentStamp: 'cwp 1/8/2005 13:09' prior: 0!
OBSourceFilesRequest is used to work around the fact that Squeak's source files are stored in a global array called SourceFiles. When testing OBMethodVersion and OBVersionBrowser, we don't want to use the real source files, as they are too unpredictable. Instead the test cases supply a source file array with known contents.

Therefore, OBMethodVersion never refers directly to the SourceFiles global. Instead it raises an OBSourceFiles request. If this occurs during a test, the test catches the notification and resumes using the test source files. Otherwise, the default action resumes using the global source file array.!

!OBSourceFilesRequest methodsFor: 'exceptionDescription' stamp: ''!
defaultAction
	^ SourceFiles! !

!OBSourceFilesRequest methodsFor: 'exceptionDescription' stamp: 'cwp 11/27/2004 22:49'!
isBrowseRequest
	^ false! !

BogusTestCase subclass: #OBActorTest
	instanceVariableNames: 'notification'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBActorTest class methodsFor: 'testing' stamp: ''!
isAbstract
	^ self name = #OBActorTest! !

!OBActorTest methodsFor: 'fetching' stamp: ''!
actionNamed: aName forNode: aNode
	| action |
	action := (self actor actionsForNode: aNode) 
				detect: [:ea | ea label = aName].
	action monitor: self.
	^ action! !

!OBActorTest methodsFor: 'fetching' stamp: ''!
actionNamed: aName forParent: aNode
	| action |
	action := (OBCategoryActor new actionsForParent: aNode) 
				detect: [:ea | ea label = aName].
	action monitor: self.
	^ action! !

!OBActorTest methodsFor: 'asserting' stamp: ''!
assertActions: actions includeLabels: expected
	| actual |
	actual := actions collect: [:ea | ea label].
	self assert: (actual includesAllOf: expected)
	! !

!OBActorTest methodsFor: 'constants' stamp: ''!
classCatNode
	^ OBClassCategoryNode on: 'Bogus-Base' inEnvironment: Smalltalk! !

!OBActorTest methodsFor: 'constants' stamp: ''!
classNode
	^ OBClassNode on: (Smalltalk at: #BogusA)! !

!OBActorTest methodsFor: 'constants' stamp: ''!
envNode
	^ OBEnvironmentNode forImage! !

!OBActorTest methodsFor: 'constants' stamp: ''!
methCatNode
	^ OBMethodCategoryNode on: 'letters' inClass: BogusA! !

!OBActorTest methodsFor: 'constants' stamp: ''!
nodeLabels
	^ #()! !

!OBActorTest methodsFor: 'emulating' stamp: 'cwp 9/17/2004 00:38'!
notification: aNotification
	notification _ aNotification! !

!OBActorTest methodsFor: 'constants' stamp: ''!
parentLabels
	^ #()! !

!OBActorTest methodsFor: 'tests' stamp: ''!
testNodeLabels
	| actions |
	actions := self actor actionsForNode: self selectedNode.
	self assertActions: actions includeLabels: self nodeLabels
	! !

!OBActorTest methodsFor: 'tests' stamp: ''!
testParentLabels
	| actions |
	actions := self actor actionsForParent: self parentNode.
	self assertActions: actions includeLabels: self parentLabels! !

OBActorTest subclass: #OBCategoryActorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBCategoryActorTest methodsFor: 'constants' stamp: ''!
actor
	^ OBCategoryActor new! !

!OBCategoryActorTest methodsFor: 'constants' stamp: ''!
nodeLabels
	^ #('remove' 'rename...').! !

!OBCategoryActorTest methodsFor: 'constants' stamp: 'cwp 11/7/2004 00:06'!
organization
	^  '(''letters'' a b)
(''numbers'' five one three two)
(''other'' bogusHopefullyUniqueSelector bogusOtherUniqueSelector isString)
(''*bogusext'' foo)
(''*bogusext-testing'' isFake)
'! !

!OBCategoryActorTest methodsFor: 'constants' stamp: 'cwp 9/22/2004 20:59'!
parentLabels
	^ #('create category...' 
		'alphabetize categories' 
		'remove empty categories' 
		'categorize automatically'
		'reorganize categories').! !

!OBCategoryActorTest methodsFor: 'constants' stamp: ''!
parentNode
	^ self classNode! !

!OBCategoryActorTest methodsFor: 'constants' stamp: ''!
selectedNode
	^ self classCatNode! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/23/2004 22:29'!
testCategorizeAll
	| classify |
	BogusA  organization classify: #beep under: (Categorizer default).
	damage add: #Categories ofClass: #BogusA.
	classify := self actionNamed: 'categorize automatically' forParent: self classNode.
	classify trigger.
	self deny: (BogusA organization categories includes: (Categorizer default)).
	self assert: (BogusA organization categoryOfElement: #beep) 
					= (Object organization categoryOfElement: #beep).
	BogusA organization removeElement: #beep	! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/23/2004 23:26'!
testCreateClassCat
	| remove |
	remove := self actionNamed: 'create category...' forParent: self envNode.
	[remove trigger]
		on: OBTextRequest
		do: [:n | self assert: n template = 'Category-Name']! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/23/2004 23:24'!
testCreateMethodCat
	| remove |
	remove := self actionNamed: 'create category...' forParent: self classNode.
	[remove trigger]
		on: OBTextRequest
		do: [:n | self assert: n template = 'category name'.
				n resume: 'zletters'].
	damage add: #Categories ofClass: #BogusA.
	self assert: (BogusA organization categories includes: #zletters)! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/18/2004 23:49'!
testNoSystemAutoCat
	| action |
	action := (OBCategoryActor new actionsForParent: self envNode) 
				detect: [:ea | ea label = 'categorize automatically'] ifNone: [nil].
	self assert: action isNil! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/18/2004 21:47'!
testRemoveEmptyMethodCats
	| remove |
	BogusA  organization addCategory: #zletters.
	damage add: #Categories ofClass: #BogusA.
	remove := self actionNamed: 'remove empty categories' forParent: self classNode.
	remove trigger.
	self deny: (BogusA organization categories includes: #zletters)! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/17/2004 00:39'!
testRemoveMethodCat
	| remove |
	remove := self actionNamed: 'remove' forNode: self methCatNode.
	[remove trigger]
		on: OBConfirmationRequest
		do: [:n | n resume: true].
	damage add: #a ofClass: #BogusA.
	damage add: #b ofClass: #BogusA.
	self deny: (BogusA organization categories includes: #letters)! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/17/2004 00:39'!
testRenameMethodCat
	| rename |
	rename := self actionNamed: 'rename...' forNode: self methCatNode.
	[rename trigger]
		on: OBTextRequest
		do: [:n | n resume: 'zletters'].
	damage add: #Categories ofClass: #BogusA.
	self deny: (BogusA organization categories includes: #letters).
	self assert: (BogusA organization categories includes: #zletters)! !

!OBCategoryActorTest methodsFor: 'tests' stamp: 'cwp 9/22/2004 20:55'!
testReorganize
	| action |
	action := self actionNamed: 'reorganize categories' forParent: self classNode.
	action trigger.
	self assert: notification definition notNil.
	self assert: notification definition text = self organization. ! !

!OBCategoryActorTest methodsFor: 'tests' stamp: ''!
testSortMethodCats
	| remove |
	BogusA organization addCategory: 'zzz' before: BogusA organization categories first.
	remove := self actionNamed: 'alphabetize categories' forParent: self classNode.
	remove trigger.
	damage add: #Categories ofClass: #BogusA.
	self assert: BogusA organization categories = BogusA organization categories asSortedArray! !

OBActorTest subclass: #OBClassActorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBClassActorTest methodsFor: 'constants' stamp: ''!
actor
	^ OBClassActor new! !

!OBClassActorTest methodsFor: 'asserting' stamp: 'cwp 11/16/2004 23:43'!
assertCorrectBrowser: aBrowser
	self assert: (aBrowser isKindOf: OBListBrowser)! !

!OBClassActorTest methodsFor: 'constants' stamp: ''!
classFNode
	^ OBClassNode on: (Smalltalk at: #BogusF)! !

!OBClassActorTest methodsFor: 'constants' stamp: 'cwp 9/16/2004 15:24'!
nodeLabels
	^ #('remove' 'rename...' 'copy...' 'subclass template')! !

!OBClassActorTest methodsFor: 'constants' stamp: 'cwp 11/30/2004 22:20'!
ownClassNode
	^ OBClassNode on: (Smalltalk at: self class name)! !

!OBClassActorTest methodsFor: 'constants' stamp: ''!
parentLabels
	^ #()! !

!OBClassActorTest methodsFor: 'constants' stamp: ''!
parentNode
	^ self classCatNode! !

!OBClassActorTest methodsFor: 'constants' stamp: ''!
selectedNode
	^ self classNode! !

!OBClassActorTest methodsFor: 'constants' stamp: 'cwp 9/17/2004 00:45'!
subclassTemplate
	^  'BogusA subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''Bogus-Base'''! !

!OBClassActorTest methodsFor: 'tests' stamp: 'cwp 10/17/2004 20:10'!
testCopy
	| rename |
	damage add: #Definition ofClass: #BogusF.
	damage add: #Definition ofClass: #BogusX.
	rename := self actionNamed: 'copy...' forNode: self classFNode.
	[rename trigger]
		on: OBTextRequest
		do: [:n | n resume: 'BogusX'].
	self assert: (Smalltalk hasClassNamed: #BogusF).
	self assert: (Smalltalk hasClassNamed: #BogusX)! !

!OBClassActorTest methodsFor: 'tests' stamp: 'cwp 12/2/2004 23:14'!
testReferences
	| remove |
	remove := self actionNamed: 'browse references' forNode: self classFNode.
	[remove trigger]
		on: OBBrowseRequest
		do: [:n | self assertCorrectBrowser: n browser].
! !

!OBClassActorTest methodsFor: 'tests' stamp: 'cwp 10/17/2004 20:10'!
testRemove
	| remove |
	damage add: #Definition ofClass: #BogusF.
	remove := self actionNamed: 'remove' forNode: self classFNode.
	[remove trigger]
		on: OBConfirmationRequest
		do: [:n | n resume: true].
	self deny: (Smalltalk hasClassNamed: #BogusF)! !

!OBClassActorTest methodsFor: 'tests' stamp: 'cwp 10/17/2004 22:35'!
testRename
	| rename |
	damage addRenameOf: #BogusF to: #BogusX.
	rename := self actionNamed: 'rename...' forNode: self classFNode.
	[rename trigger]
		on: OBInteractionRequest
		do: [:n | n resume: 'BogusX'].
	self deny: (Smalltalk hasClassNamed: #BogusF).
	self assert: (Smalltalk hasClassNamed: #BogusX)! !

!OBClassActorTest methodsFor: 'tests' stamp: 'cwp 11/30/2004 22:13'!
testRenamedReferences
	| rename raised |
	damage addRenameOf: #BogusA to: #BogusX.
	raised _ false.
	rename := self actionNamed: 'rename...' forNode: self classNode.
	[rename trigger] 
		on: OBTextRequest, OBBrowseRequest
		do: [:n | n isBrowseRequest	
					ifTrue: [self assertCorrectBrowser: n browser.
							raised _ true]
					ifFalse: [n resume: 'BogusX']].
	self assert: raised.! !

!OBClassActorTest methodsFor: 'tests' stamp: 'cwp 12/2/2004 22:43'!
testRenameNoReferences
	| rename newName |
	newName := self class name, 'X'.
	damage addRenameOf: self class name to: newName asSymbol.
	rename := self actionNamed: 'rename...' forNode: self ownClassNode.
	self 
		shouldnt: [[rename trigger] 
					on: OBTextRequest
					do: [:n | n resume: newName]]
		raise: OBBrowseRequest.
! !

!OBClassActorTest methodsFor: 'tests' stamp: 'cwp 9/17/2004 00:47'!
testSubclassTemplate
	| action |
	action := self actionNamed: 'subclass template' forNode: self classNode.
	action trigger.
	self assert: notification definition notNil.
	self assert: notification definition text = self subclassTemplate. ! !

BogusTestCase subclass: #OBOrganizationDefinitionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Tests'!

!OBOrganizationDefinitionTest methodsFor: 'constants' stamp: 'cwp 9/22/2004 22:08'!
definition
	^ OBOrganizationDefinition on: BogusA organization! !

!OBOrganizationDefinitionTest methodsFor: 'constants' stamp: 'cwp 11/6/2004 23:18'!
newOrgText
	^ '(''alphanum'' a b five one three two)
(''other'' bogusHopefullyUniqueSelector bogusOtherUniqueSelector isString)
(''*bogusext'' foo)
(''*bogusext-testing'' isFake)
'
! !

!OBOrganizationDefinitionTest methodsFor: 'constants' stamp: 'cwp 11/6/2004 23:17'!
orgText
	^ '(''letters'' a b)
(''numbers'' five one three two)
(''other'' bogusHopefullyUniqueSelector bogusOtherUniqueSelector isString)
(''*bogusext'' foo)
(''*bogusext-testing'' isFake)
'! !

!OBOrganizationDefinitionTest methodsFor: 'tests' stamp: 'cwp 9/23/2004 22:00'!
testAccept
	| definition |
	definition _ self definition.
	definition accept: self newOrgText notifying: self.
	damage add: #Categories ofClass: #BogusA.
	self assert: definition text = self newOrgText.
	self assert: BogusA organization asString = self newOrgText! !

!OBOrganizationDefinitionTest methodsFor: 'tests' stamp: 'cwp 9/22/2004 22:31'!
testDisplay
	self assert: self definition text = self orgText! !

!SystemOrganizer methodsFor: '*ob-standard-testing' stamp: 'cwp 9/18/2004 23:58'!
isClassOrganizer
	^ false! !
OBSystemBrowserAdaptor initialize!
OBSystemBrowser initialize!
